@@ -18,21 +18,30 @@ namespace ixion {
 class IXION_DLLPUBLIC general_error : public std::exception
 {
 public:
+    general_error();
     explicit general_error(const std::string& msg);
-    ~general_error() throw();
-    virtual const char* what() const throw();
+    virtual ~general_error() throw();
+    virtual const char* what() const throw() override;
+
+protected:
+    void set_message(const std::string& msg);
+
 private:
     std::string m_msg;
 };
 
-class IXION_DLLPUBLIC file_not_found : public std::exception
+class IXION_DLLPUBLIC file_not_found : public general_error
 {
 public:
     explicit file_not_found(const std::string& fpath);
-    ~file_not_found() throw();
-    virtual const char* what() const throw();
-private:
-    std::string m_fpath;
+    virtual ~file_not_found() throw() override;
+};
+
+class IXION_DLLPUBLIC formula_registration_error : public general_error
+{
+public:
+    explicit formula_registration_error(const std::string& msg);
+    virtual ~formula_registration_error() throw() override;
 };
 
 /**
@@ -44,10 +53,13 @@ public:
     enum error_type
     {
         circular_dependency,
-        sheet_name_conflict
+        sheet_name_conflict,
+        sheet_size_locked,
+        not_implemented
     };
 
     explicit model_context_error(const std::string& msg, error_type type);
+    virtual ~model_context_error() throw() override;
 
     error_type get_error_type() const;
 
@@ -55,6 +67,13 @@ private:
     error_type m_type;
 };
 
+class IXION_DLLPUBLIC not_implemented_error : public general_error
+{
+public:
+    explicit not_implemented_error(const std::string& msg);
+    virtual ~not_implemented_error() throw() override;
+};
+
 }
 
 #endif
