@@ -10,9 +10,9 @@
 #include <string>
 #include <vector>
 #include <iostream>
+#include <thread>
 
 #include <boost/program_options.hpp>
-#include <thread>
 
 using namespace std;
 using namespace ixion;
@@ -49,6 +49,15 @@ public:
     }
 };
 
+const char* help_thread =
+"Specify the number of threads to use for calculation.  Note that the number "
+"specified by this option corresponds with the number of calculation threads "
+"i.e. those child threads that perform cell interpretations. The main thread "
+"does not perform any calculations; instead, it creates a new child thread to "
+"manage the calculation threads, the number of which is specified by the arg. "
+"Therefore, the total number of threads used by this program will be arg + 1."
+;
+
 }
 
 int main (int argc, char** argv)
@@ -58,13 +67,12 @@ int main (int argc, char** argv)
     size_t thread_count = 0;
     po::options_description desc("Allowed options");
     desc.add_options()
-        ("help,h", "print this help.")
-        ("thread,t", po::value<size_t>(),
-         "specify the number of threads to use for calculation.  Note that the number specified by this option corresponds with the number of calculation threads i.e. those child threads that perform cell interpretations.  The main thread does not perform any calculations; instead, it creates a new child thread to manage the calculation threads, the number of which is specified by the arg.  Therefore, the total number of threads used by this program will be arg + 1.");
+        ("help,h", "Print this help.")
+        ("thread,t", po::value<size_t>(), help_thread);
 
     po::options_description hidden("Hidden options");
     hidden.add_options()
-        ("input-file", po::value< vector<string> >(), "input file");
+        ("input-file", po::value<std::vector<std::string>>(), "input file");
 
     po::options_description cmd_opt;
     cmd_opt.add(desc).add(hidden);
@@ -96,6 +104,8 @@ int main (int argc, char** argv)
         return EXIT_SUCCESS;
     }
 
+    ixion::init();
+
     if (vm.count("thread"))
         thread_count = vm["thread"].as<size_t>();
 
