@@ -7,7 +7,6 @@
 
 #include "ixion/cell.hpp"
 #include "ixion/address.hpp"
-#include "ixion/cell_listener_tracker.hpp"
 #include "ixion/exceptions.hpp"
 #include "ixion/formula_result.hpp"
 #include "ixion/formula_tokens.hpp"
@@ -15,8 +14,11 @@
 #include "ixion/interface/session_handler.hpp"
 #include "ixion/global.hpp"
 #include "ixion/matrix.hpp"
+#include "ixion/formula_name_resolver.hpp"
+#include "ixion/formula.hpp"
 
 #include "formula_interpreter.hpp"
+#include "debug.hpp"
 
 #include <cassert>
 #include <string>
@@ -25,11 +27,6 @@
 #include <algorithm>
 #include <functional>
 
-#define DEBUG_FORMULA_CELL 0
-#if DEBUG_FORMULA_CELL
-#include "ixion/formula_name_resolver.hpp"
-#endif
-
 #include "calc_status.hpp"
 
 #define FORMULA_CIRCULAR_SAFE 0x01
@@ -39,14 +36,28 @@ using namespace std;
 
 namespace ixion {
 
+namespace {
+
+#if IXION_LOGGING
+
+std::string gen_trace_output(const formula_cell& fc, const iface::formula_model_access& cxt, const abs_address_t& pos)
+{
+    auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
+    std::ostringstream os;
+    os << "pos=" << pos.get_name() << "; formula='" << print_formula_tokens(cxt, pos, *resolver, fc.get_tokens()->get()) << "'";
+    return os.str();
+}
+
+#endif
+
+}
+
 struct formula_cell::impl
 {
     mutable calc_status_ptr_t m_calc_status;
     formula_tokens_store_ptr_t m_tokens;
     rc_address_t m_group_pos;
 
-    bool m_circular_safe:1;
-
     impl() : impl(-1, -1, new calc_status, formula_tokens_store_ptr_t()) {}
 
     impl(const formula_tokens_store_ptr_t& tokens) : impl(-1, -1, new calc_status, tokens) {}
@@ -55,8 +66,7 @@ struct formula_cell::impl
         const formula_tokens_store_ptr_t& tokens) :
         m_calc_status(cs),
         m_tokens(tokens),
-        m_group_pos(row, col, false, false),
-        m_circular_safe(false) {}
+        m_group_pos(row, col, false, false) {}
 
     /**
      * Block until the result becomes available.
@@ -65,21 +75,18 @@ struct formula_cell::impl
      */
     void wait_for_interpreted_result(std::unique_lock<std::mutex>& lock) const
     {
-#if DEBUG_FORMULA_CELL
-        __IXION_DEBUG_OUT__ << "wait for interpreted result" << endl;
-#endif
+        IXION_TRACE("Wait for the interpreted result");
+
         while (!m_calc_status->result)
         {
-#if DEBUG_FORMULA_CELL
-            __IXION_DEBUG_OUT__ << "waiting" << endl;
-#endif
+            IXION_TRACE("Waiting...");
             m_calc_status->cond.wait(lock);
         }
     }
 
     void reset_flag()
     {
-        m_circular_safe = false;
+        m_calc_status->circular_safe = false;
     }
 
     /**
@@ -90,7 +97,7 @@ struct formula_cell::impl
      */
     bool is_circular_safe() const
     {
-        return m_circular_safe;
+        return m_calc_status->circular_safe;
     }
 
     bool check_ref_for_circular_safety(const formula_cell& ref, const abs_address_t& pos)
@@ -98,9 +105,7 @@ struct formula_cell::impl
         if (!ref.mp_impl->is_circular_safe())
         {
             // Circular dependency detected !!
-#if DEBUG_FORMULA_CELL
-            __IXION_DEBUG_OUT__ << "circular dependency detected !!" << endl;
-#endif
+            IXION_DEBUG("Circular dependency detected !!");
             assert(!m_calc_status->result);
             m_calc_status->result =
                 ixion::make_unique<formula_result>(formula_error_t::ref_result_not_available);
@@ -110,15 +115,26 @@ struct formula_cell::impl
         return true;
     }
 
-    double fetch_value_from_result() const
+    void check_calc_status_or_throw() const
     {
         if (!m_calc_status->result)
+        {
             // Result not cached yet.  Reference error.
+            IXION_DEBUG("Result not cached yet. This is a reference error.");
             throw formula_error(formula_error_t::ref_result_not_available);
+        }
 
         if (m_calc_status->result->get_type() == formula_result::result_type::error)
+        {
             // Error condition.
+            IXION_DEBUG("Error in result.");
             throw formula_error(m_calc_status->result->get_error());
+        }
+    }
+
+    double fetch_value_from_result() const
+    {
+        check_calc_status_or_throw();
 
         switch (m_calc_status->result->get_type())
         {
@@ -126,6 +142,7 @@ struct formula_cell::impl
                 return m_calc_status->result->get_value();
             case formula_result::result_type::matrix:
             {
+                IXION_TRACE("Fetching a matrix result.");
                 const matrix& m = m_calc_status->result->get_matrix();
                 row_t row_size = m.row_size();
                 col_t col_size = m.col_size();
@@ -144,15 +161,70 @@ struct formula_cell::impl
                     case matrix::element_type::boolean:
                         return elem.boolean ? 1.0 : 0.0;
                     case matrix::element_type::string:
+                    case matrix::element_type::error:
                     default:
                         throw formula_error(formula_error_t::invalid_value_type);
                 }
             }
             default:
-                throw formula_error(formula_error_t::invalid_value_type);
+            {
+                std::ostringstream os;
+                os << "numeric result was requested, but the actual result is of "
+                    << m_calc_status->result->get_type() << " type.";
+                throw formula_error(
+                    formula_error_t::invalid_value_type,
+                    os.str()
+                );
+            }
         }
     }
 
+    const std::string* fetch_string_from_result() const
+    {
+        check_calc_status_or_throw();
+
+        switch (m_calc_status->result->get_type())
+        {
+            case formula_result::result_type::string:
+                return &m_calc_status->result->get_string();
+            case formula_result::result_type::matrix:
+            {
+                const matrix& m = m_calc_status->result->get_matrix();
+                row_t row_size = m.row_size();
+                col_t col_size = m.col_size();
+
+                if (m_group_pos.row >= row_size || m_group_pos.column >= col_size)
+                    throw formula_error(formula_error_t::invalid_value_type);
+
+                matrix::element elem = m.get(m_group_pos.row, m_group_pos.column);
+
+                switch (elem.type)
+                {
+                    case matrix::element_type::string:
+                        return elem.str;
+                    case matrix::element_type::numeric:
+                    case matrix::element_type::empty:
+                    case matrix::element_type::boolean:
+                    case matrix::element_type::error:
+                    default:
+                        throw formula_error(formula_error_t::invalid_value_type);
+                }
+            }
+            default:
+            {
+                std::ostringstream os;
+                os << "string result was requested, but the actual result is of "
+                    << m_calc_status->result->get_type() << " type.";
+                throw formula_error(
+                    formula_error_t::invalid_value_type,
+                    os.str()
+                );
+            }
+        }
+
+        return nullptr;
+    }
+
     bool is_grouped() const
     {
         return m_group_pos.column >= 0 && m_group_pos.row >= 0;
@@ -196,7 +268,9 @@ struct formula_cell::impl
             case matrix::element_type::numeric:
                 return formula_result(elem.numeric);
             case matrix::element_type::string:
-                return formula_result(elem.string_id);
+                return formula_result(*elem.str);
+            case matrix::element_type::error:
+                return formula_result(elem.error);
             case matrix::element_type::empty:
                 return formula_result();
             case matrix::element_type::boolean:
@@ -205,6 +279,45 @@ struct formula_cell::impl
                 throw std::logic_error("unhandled element type of a matrix result value.");
         }
     }
+
+    void set_single_formula_result(formula_result result)
+    {
+        if (is_grouped())
+        {
+            std::unique_lock<std::mutex> lock(m_calc_status->mtx);
+
+            if (!m_calc_status->result)
+            {
+                m_calc_status->result =
+                    ixion::make_unique<formula_result>(
+                        matrix(m_calc_status->group_size.row, m_calc_status->group_size.column));
+            }
+
+            matrix& m = m_calc_status->result->get_matrix();
+            assert(m_group_pos.row < row_t(m.row_size()));
+            assert(m_group_pos.column < col_t(m.col_size()));
+
+            switch (result.get_type())
+            {
+                case formula_result::result_type::value:
+                    m.set(m_group_pos.row, m_group_pos.column, result.get_value());
+                    break;
+                case formula_result::result_type::string:
+                    m.set(m_group_pos.row, m_group_pos.column, result.get_string());
+                    break;
+                case formula_result::result_type::error:
+                    m.set(m_group_pos.row, m_group_pos.column, result.get_error());
+                    break;
+                case formula_result::result_type::matrix:
+                    throw std::logic_error("setting a cached result of matrix value directly is not yet supported.");
+            }
+
+            return;
+        }
+
+        std::unique_lock<std::mutex> lock(m_calc_status->mtx);
+        m_calc_status->result = ixion::make_unique<formula_result>(std::move(result));
+    }
 };
 
 formula_cell::formula_cell() : mp_impl(ixion::make_unique<impl>()) {}
@@ -232,25 +345,26 @@ void formula_cell::set_tokens(const formula_tokens_store_ptr_t& tokens)
     mp_impl->m_tokens = tokens;
 }
 
-double formula_cell::get_value() const
+double formula_cell::get_value(formula_result_wait_policy_t policy) const
 {
     std::unique_lock<std::mutex> lock(mp_impl->m_calc_status->mtx);
-    mp_impl->wait_for_interpreted_result(lock);
+    if (policy == formula_result_wait_policy_t::block_until_done)
+        mp_impl->wait_for_interpreted_result(lock);
     return mp_impl->fetch_value_from_result();
 }
 
-double formula_cell::get_value_nowait() const
+const std::string* formula_cell::get_string(formula_result_wait_policy_t policy) const
 {
-    std::lock_guard<std::mutex> lock(mp_impl->m_calc_status->mtx);
-    return mp_impl->fetch_value_from_result();
+    std::unique_lock<std::mutex> lock(mp_impl->m_calc_status->mtx);
+    if (policy == formula_result_wait_policy_t::block_until_done)
+        mp_impl->wait_for_interpreted_result(lock);
+    return mp_impl->fetch_string_from_result();
 }
 
 void formula_cell::interpret(iface::formula_model_access& context, const abs_address_t& pos)
 {
-#if DEBUG_FORMULA_CELL
-    const formula_name_resolver& resolver = context.get_name_resolver();
-    __IXION_DEBUG_OUT__ << resolver.get_name(pos, false) << ": interpreting" << endl;
-#endif
+    IXION_TRACE(gen_trace_output(*this, context, pos));
+
     if (!mp_impl->calc_allowed())
         throw std::logic_error("Calculation on this formula cell is not allowed.");
 
@@ -321,9 +435,25 @@ void formula_cell::check_circular(const iface::formula_model_access& cxt, const
                 abs_range_t range = t->get_range_ref().to_abs(pos);
                 for (sheet_t sheet = range.first.sheet; sheet <= range.last.sheet; ++sheet)
                 {
-                    for (col_t col = range.first.column; col <= range.last.column; ++col)
+                    rc_size_t sheet_size = cxt.get_sheet_size();
+                    col_t col_first = range.first.column, col_last = range.last.column;
+                    if (range.all_columns())
                     {
-                        for (row_t row = range.first.row; row <= range.last.row; ++row)
+                        col_first = 0;
+                        col_last = sheet_size.column - 1;
+                    }
+
+                    for (col_t col = col_first; col <= col_last; ++col)
+                    {
+                        row_t row_first = range.first.row, row_last = range.last.row;
+                        if (range.all_rows())
+                        {
+                            assert(row_last == row_unset);
+                            row_first = 0;
+                            row_last = sheet_size.row - 1;
+                        }
+
+                        for (row_t row = row_first; row <= row_last; ++row)
                         {
                             abs_address_t addr(sheet, row, col);
                             if (cxt.get_celltype(addr) != celltype_t::formula)
@@ -338,18 +468,13 @@ void formula_cell::check_circular(const iface::formula_model_access& cxt, const
                 break;
             }
             default:
-#if DEBUG_FORMULA_CELL
-                __IXION_DEBUG_OUT__ << "check_circular: token type " << get_opcode_name((*itr)->get_opcode())
-                    << " was not processed." << endl;
-#else
                 ;
-#endif
         }
 
     }
 
     // No circular dependencies.  Good.
-    mp_impl->m_circular_safe = true;
+    mp_impl->m_calc_status->circular_safe = true;
 }
 
 void formula_cell::reset()
@@ -374,7 +499,7 @@ std::vector<const formula_token*> formula_cell::get_ref_tokens(
                 break;
             case fop_named_expression:
             {
-                const formula_tokens_t* named_exp =
+                const named_expression_t* named_exp =
                     cxt.get_named_expression(pos.sheet, t->get_name());
 
                 if (!named_exp)
@@ -382,7 +507,7 @@ std::vector<const formula_token*> formula_cell::get_ref_tokens(
                     break;
 
                 // recursive call.
-                std::for_each(named_exp->begin(), named_exp->end(), get_refs);
+                std::for_each(named_exp->tokens.begin(), named_exp->tokens.end(), get_refs);
                 break;
             }
             default:
@@ -397,35 +522,31 @@ std::vector<const formula_token*> formula_cell::get_ref_tokens(
     return ret;
 }
 
-const formula_result& formula_cell::get_raw_result_cache() const
+const formula_result& formula_cell::get_raw_result_cache(formula_result_wait_policy_t policy) const
 {
     std::unique_lock<std::mutex> lock(mp_impl->m_calc_status->mtx);
-    mp_impl->wait_for_interpreted_result(lock);
+
+    if (policy == formula_result_wait_policy_t::block_until_done)
+        mp_impl->wait_for_interpreted_result(lock);
+
     if (!mp_impl->m_calc_status->result)
+    {
+        IXION_DEBUG("Result not yet available.");
         throw formula_error(formula_error_t::ref_result_not_available);
+    }
 
     return *mp_impl->m_calc_status->result;
 }
 
-const formula_result* formula_cell::get_raw_result_cache_nowait() const
+formula_result formula_cell::get_result_cache(formula_result_wait_policy_t policy) const
 {
-    std::unique_lock<std::mutex> lock(mp_impl->m_calc_status->mtx);
-    return mp_impl->m_calc_status->result.get();
-}
-
-formula_result formula_cell::get_result_cache() const
-{
-    const formula_result& src = get_raw_result_cache();
+    const formula_result& src = get_raw_result_cache(policy);
     return mp_impl->get_single_formula_result(src);
 }
 
-formula_result formula_cell::get_result_cache_nowait() const
+void formula_cell::set_result_cache(formula_result result)
 {
-    const formula_result* src = get_raw_result_cache_nowait();
-    if (!src)
-        return formula_result(formula_error_t::no_result_error);
-
-    return mp_impl->get_single_formula_result(*src);
+    mp_impl->set_single_formula_result(result);
 }
 
 formula_group_t formula_cell::get_group_properties() const
