@@ -28,6 +28,15 @@ void general_error::set_message(const std::string& msg)
     m_msg = msg;
 }
 
+formula_registration_error::formula_registration_error(const std::string& msg)
+{
+    std::ostringstream os;
+    os << "formula_registration_error: " << msg;
+    set_message(os.str());
+}
+
+formula_registration_error::~formula_registration_error() {}
+
 file_not_found::file_not_found(const std::string& fpath) :
     general_error(fpath)
 {
@@ -48,6 +57,15 @@ model_context_error::error_type model_context_error::get_error_type() const
     return m_type;
 }
 
+not_implemented_error::not_implemented_error(const std::string& msg)
+{
+    std::ostringstream os;
+    os << "not_implemented_error: " << msg;
+    set_message(os.str());
+}
+
+not_implemented_error::~not_implemented_error() {}
+
 }
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
