@@ -8,21 +8,22 @@
 #include "formula_interpreter.hpp"
 #include "formula_functions.hpp"
 #include "concrete_formula_tokens.hpp"
+#include "debug.hpp"
 
 #include "ixion/cell.hpp"
 #include "ixion/global.hpp"
-#include "ixion/formula_name_resolver.hpp"
 #include "ixion/matrix.hpp"
+#include "ixion/formula.hpp"
 #include "ixion/interface/formula_model_access.hpp"
 #include "ixion/interface/session_handler.hpp"
 #include "ixion/interface/table_handler.hpp"
+#include "ixion/config.hpp"
 
 #include <cassert>
 #include <string>
 #include <iostream>
 #include <sstream>
-
-#define DEBUG_FORMULA_INTERPRETER 0
+#include <cmath>
 
 using namespace std;
 
@@ -44,7 +45,6 @@ opcode_token paren_close = opcode_token(fop_close);
 formula_interpreter::formula_interpreter(const formula_cell* cell, iface::formula_model_access& cxt) :
     m_parent_cell(cell),
     m_context(cxt),
-    m_stack(cxt),
     m_error(formula_error_t::no_error)
 {
 }
@@ -70,9 +70,7 @@ bool formula_interpreter::interpret()
 
         if (m_tokens.empty())
         {
-#if DEBUG_FORMULA_INTERPRETER
-            __IXION_DEBUG_OUT__ << "interpreter error: no tokens to interpret" << endl;
-#endif
+            IXION_DEBUG("interpreter has no tokens to interpret");
             return false;
         }
 
@@ -81,17 +79,18 @@ bool formula_interpreter::interpret()
         m_result.reset();
 
         expression();
+
         if (m_cur_token_itr != m_tokens.end())
         {
             if (mp_handler)
                 mp_handler->set_invalid_expression("formula token interpretation ended prematurely.");
             return false;
         }
+
         pop_result();
 
-#if DEBUG_FORMULA_INTERPRETER
-        __IXION_DEBUG_OUT__ << "interpretation successfully finished" << endl;
-#endif
+        IXION_TRACE("interpretation successfully finished");
+
         if (mp_handler)
             mp_handler->end_cell_interpret();
 
@@ -106,9 +105,8 @@ bool formula_interpreter::interpret()
     }
     catch (const formula_error& e)
     {
-#if DEBUG_FORMULA_INTERPRETER
-        __IXION_DEBUG_OUT__ << "formula error: " << e.what() << endl;
-#endif
+        IXION_DEBUG("formula error: " << e.what());
+
         if (mp_handler)
             mp_handler->set_formula_error(e.what());
 
@@ -133,9 +131,10 @@ formula_error_t formula_interpreter::get_error() const
 
 void formula_interpreter::init_tokens()
 {
+    clear_stacks();
+
     name_set used_names;
     m_tokens.clear();
-    m_stack.clear();
 
     const formula_tokens_store_ptr_t& ts = m_parent_cell->get_tokens();
     if (!ts)
@@ -148,8 +147,9 @@ void formula_interpreter::init_tokens()
         if (p->get_opcode() == fop_named_expression)
         {
             // Named expression.  Expand it.
-            const formula_tokens_t* expr = m_context.get_named_expression(
+            const named_expression_t* expr = m_context.get_named_expression(
                 m_pos.sheet, p->get_name());
+
             used_names.insert(p->get_name());
             expand_named_expression(expr, used_names);
         }
@@ -169,11 +169,7 @@ void get_result_from_cell(const iface::formula_model_access& cxt, const abs_addr
     {
         case celltype_t::formula:
         {
-            const formula_cell* fcell = cxt.get_formula_cell(addr);
-            if (!fcell)
-                return;
-
-            res = fcell->get_result_cache();
+            res = cxt.get_formula_result(addr);
             break;
         }
         case celltype_t::boolean:
@@ -184,8 +180,12 @@ void get_result_from_cell(const iface::formula_model_access& cxt, const abs_addr
             res.set_value(cxt.get_numeric_value(addr));
             break;
         case celltype_t::string:
-            res.set_string(cxt.get_string_identifier(addr));
+        {
+            const std::string* p = cxt.get_string_value(addr);
+            assert(p);
+            res.set_string_value(*p);
             break;
+        }
         case celltype_t::unknown:
         default:
             ;
@@ -197,8 +197,8 @@ void get_result_from_cell(const iface::formula_model_access& cxt, const abs_addr
 void formula_interpreter::pop_result()
 {
     // there should only be one stack value left for the result value.
-    assert(m_stack.size() == 1);
-    stack_value& res = m_stack.back();
+    assert(get_stack().size() == 1);
+    stack_value& res = get_stack().back();
     switch (res.get_type())
     {
         case stack_value_t::range_ref:
@@ -213,12 +213,12 @@ void formula_interpreter::pop_result()
             break;
         }
         case stack_value_t::string:
-            m_result.set_string(res.get_string());
+        {
+            m_result.set_string_value(res.get_string());
             break;
+        }
         case stack_value_t::value:
-#if DEBUG_FORMULA_INTERPRETER
-            __IXION_DEBUG_OUT__ << "result: " << res.get_value() << endl;
-#endif
+            IXION_TRACE("value=" << res.get_value());
             m_result.set_value(res.get_value());
             break;
         case stack_value_t::matrix:
@@ -232,16 +232,15 @@ void formula_interpreter::pop_result()
         mp_handler->set_result(m_result);
 }
 
-void formula_interpreter::expand_named_expression(const formula_tokens_t* expr, name_set& used_names)
+void formula_interpreter::expand_named_expression(const named_expression_t* expr, name_set& used_names)
 {
     if (!expr)
         throw formula_error(formula_error_t::name_not_found);
 
     m_tokens.push_back(&paren_open);
-    formula_tokens_t::const_iterator itr = expr->begin(), itr_end = expr->end();
-    for (; itr != itr_end; ++itr)
+    for (const auto& token : expr->tokens)
     {
-        const formula_token& t = **itr;
+        const formula_token& t = *token;
         if (t.get_opcode() == fop_named_expression)
         {
             string expr_name = t.get_name();
@@ -250,7 +249,7 @@ void formula_interpreter::expand_named_expression(const formula_tokens_t* expr,
                 // Circular reference detected.
                 throw invalid_expression("circular referencing of named expressions");
             }
-            const formula_tokens_t* this_expr = m_context.get_named_expression(m_pos.sheet, expr_name);
+            const named_expression_t* this_expr = m_context.get_named_expression(m_pos.sheet, expr_name);
             used_names.insert(expr_name);
             expand_named_expression(this_expr, used_names);
         }
@@ -260,6 +259,12 @@ void formula_interpreter::expand_named_expression(const formula_tokens_t* expr,
     m_tokens.push_back(&paren_close);
 }
 
+void formula_interpreter::ensure_token_exists() const
+{
+    if (!has_token())
+        throw invalid_expression("formula expression ended prematurely");
+}
+
 bool formula_interpreter::has_token() const
 {
     return m_cur_token_itr != m_end_token_pos;
@@ -276,6 +281,12 @@ const formula_token& formula_interpreter::token() const
     return *(*m_cur_token_itr);
 }
 
+const formula_token& formula_interpreter::token_or_throw() const
+{
+    ensure_token_exists();
+    return *(*m_cur_token_itr);
+}
+
 const formula_token& formula_interpreter::next_token()
 {
     next();
@@ -307,17 +318,17 @@ bool valid_expression_op(fopcode_t oc)
 }
 
 bool pop_stack_value_or_string(const iface::formula_model_access& cxt,
-    value_stack_t& stack, stack_value_t& vt, double& val, string& str)
+    formula_value_stack& stack, stack_value_t& vt, double& val, string& str)
 {
     vt = stack.get_type();
     switch (vt)
     {
         case stack_value_t::value:
             val = stack.pop_value();
-        break;
+            break;
         case stack_value_t::string:
             str = stack.pop_string();
-        break;
+            break;
         case stack_value_t::single_ref:
         {
             const abs_address_t& addr = stack.pop_single_ref();
@@ -353,9 +364,7 @@ bool pop_stack_value_or_string(const iface::formula_model_access& cxt,
                 }
                 case celltype_t::formula:
                 {
-                    const formula_cell* fc = cxt.get_formula_cell(addr);
-                    assert(fc);
-                    formula_result res = fc->get_result_cache();
+                    formula_result res = cxt.get_formula_result(addr);
 
                     switch (res.get_type())
                     {
@@ -368,11 +377,7 @@ bool pop_stack_value_or_string(const iface::formula_model_access& cxt,
                         case formula_result::result_type::string:
                         {
                             vt = stack_value_t::string;
-                            string_id_t strid = res.get_string();
-                            const string* ps = cxt.get_string(strid);
-                            if (!ps)
-                                return false;
-                            str = *ps;
+                            str = res.get_string();
                             return true;
                         }
                         case formula_result::result_type::error:
@@ -383,8 +388,8 @@ bool pop_stack_value_or_string(const iface::formula_model_access& cxt,
                 default:
                     return false;
             }
+            break;
         }
-        break;
         case stack_value_t::range_ref:
         default:
             return false;
@@ -392,7 +397,7 @@ bool pop_stack_value_or_string(const iface::formula_model_access& cxt,
     return true;
 }
 
-void compare_values(value_stack_t& vs, fopcode_t oc, double val1, double val2)
+void compare_values(formula_value_stack& vs, fopcode_t oc, double val1, double val2)
 {
     switch (oc)
     {
@@ -425,7 +430,7 @@ void compare_values(value_stack_t& vs, fopcode_t oc, double val1, double val2)
     }
 }
 
-void compare_strings(value_stack_t& vs, fopcode_t oc, const std::string& str1, const std::string& str2)
+void compare_strings(formula_value_stack& vs, fopcode_t oc, const std::string& str1, const std::string& str2)
 {
     switch (oc)
     {
@@ -456,7 +461,7 @@ void compare_strings(value_stack_t& vs, fopcode_t oc, const std::string& str1, c
 }
 
 void compare_value_to_string(
-    value_stack_t& vs, fopcode_t oc, double /*val1*/, const std::string& /*str2*/)
+    formula_value_stack& vs, fopcode_t oc, double /*val1*/, const std::string& /*str2*/)
 {
     // Value 1 is numeric while value 2 is string.  String is
     // always greater than numeric value.
@@ -487,7 +492,7 @@ void compare_value_to_string(
 }
 
 void compare_string_to_value(
-    value_stack_t& vs, fopcode_t oc, const std::string& /*str1*/, double /*val2*/)
+    formula_value_stack& vs, fopcode_t oc, const std::string& /*str1*/, double /*val2*/)
 {
     switch (oc)
     {
@@ -534,7 +539,7 @@ void formula_interpreter::expression()
         bool is_val1 = true, is_val2 = true;
 
         stack_value_t vt;
-        if (!pop_stack_value_or_string(m_context, m_stack, vt, val1, str1))
+        if (!pop_stack_value_or_string(m_context, get_stack(), vt, val1, str1))
             throw formula_error(formula_error_t::general_error);
         is_val1 = vt == stack_value_t::value;
 
@@ -544,7 +549,7 @@ void formula_interpreter::expression()
         next();
         term();
 
-        if (!pop_stack_value_or_string(m_context, m_stack, vt, val2, str2))
+        if (!pop_stack_value_or_string(m_context, get_stack(), vt, val2, str2))
             throw formula_error(formula_error_t::general_error);
         is_val2 = vt == stack_value_t::value;
 
@@ -553,11 +558,11 @@ void formula_interpreter::expression()
             if (is_val2)
             {
                 // Both are numeric values.
-                compare_values(m_stack, oc, val1, val2);
+                compare_values(get_stack(), oc, val1, val2);
             }
             else
             {
-                compare_value_to_string(m_stack, oc, val1, str2);
+                compare_value_to_string(get_stack(), oc, val1, str2);
             }
         }
         else
@@ -565,12 +570,12 @@ void formula_interpreter::expression()
             if (is_val2)
             {
                 // Value 1 is string while value 2 is numeric.
-                compare_string_to_value(m_stack, oc, str1, val2);
+                compare_string_to_value(get_stack(), oc, str1, val2);
             }
             else
             {
                 // Both are strings.
-                compare_strings(m_stack, oc, str1, str2);
+                compare_strings(get_stack(), oc, str1, str2);
             }
         }
     }
@@ -593,9 +598,34 @@ void formula_interpreter::term()
                 mp_handler->push_token(oc);
 
             next();
-            double val = m_stack.pop_value();
+            double val = get_stack().pop_value();
             term();
-            m_stack.push_value(val*m_stack.pop_value());
+            get_stack().push_value(val*get_stack().pop_value());
+            return;
+        }
+        case fop_exponent:
+        {
+            if (mp_handler)
+                mp_handler->push_token(oc);
+
+            next();
+            double base = get_stack().pop_value();
+            term();
+            double exp = get_stack().pop_value();
+            get_stack().push_value(std::pow(base, exp));
+            return;
+        }
+        case fop_concat:
+        {
+            if (mp_handler)
+                mp_handler->push_token(oc);
+
+            next();
+            std::string s1 = get_stack().pop_string();
+            term();
+            std::string s2 = get_stack().pop_string();
+            std::string s = s1 + s2;
+            get_stack().push_string(std::move(s));
             return;
         }
         case fop_divide:
@@ -604,12 +634,12 @@ void formula_interpreter::term()
                 mp_handler->push_token(oc);
 
             next();
-            double val = m_stack.pop_value();
+            double val = get_stack().pop_value();
             term();
-            double val2 = m_stack.pop_value();
+            double val2 = get_stack().pop_value();
             if (val2 == 0.0)
                 throw formula_error(formula_error_t::division_by_zero);
-            m_stack.push_value(val/val2);
+            get_stack().push_value(val/val2);
             return;
         }
         default:
@@ -621,43 +651,80 @@ void formula_interpreter::factor()
 {
     // <constant> || <variable> || '(' <expression> ')' || <function>
 
+    bool negative_sign = sign(); // NB: may be precedeed by a '+' or '-' sign.
     fopcode_t oc = token().get_opcode();
+
     switch (oc)
     {
         case fop_open:
             paren();
-            return;
+            break;
         case fop_named_expression:
-#if DEBUG_FORMULA_INTERPRETER
-            __IXION_DEBUG_OUT__ << "named expression encountered in factor" << endl;
-#endif
+        {
             // All named expressions are supposed to be expanded prior to interpretation.
+            IXION_DEBUG("named expression encountered in factor.");
             throw formula_error(formula_error_t::general_error);
+        }
         case fop_value:
+        {
             constant();
-            return;
+            break;
+        }
         case fop_single_ref:
             single_ref();
-            return;
+            break;
         case fop_range_ref:
             range_ref();
-            return;
+            break;
         case fop_table_ref:
             table_ref();
-            return;
+            break;
         case fop_function:
             function();
-            return;
+            break;
         case fop_string:
             literal();
-            return;
+            break;
+        default:
+            ostringstream os;
+            os << "factor: unexpected token type: <" << get_opcode_name(oc) << ">";
+            throw invalid_expression(os.str());
+    }
+
+    if (negative_sign)
+    {
+        double v = get_stack().pop_value();
+        get_stack().push_value(v * -1.0);
+    }
+}
+
+bool formula_interpreter::sign()
+{
+    ensure_token_exists();
+
+    fopcode_t oc = token().get_opcode();
+    bool sign_set = false;
+
+    switch (oc)
+    {
+        case fop_minus:
+            sign_set = true;
+            // fall through
+        case fop_plus:
+        {
+            if (mp_handler)
+                mp_handler->push_token(oc);
+
+            next();
+
+            if (!has_token())
+                throw invalid_expression("sign: a sign cannot be the last token");
+        }
         default:
             ;
     }
 
-    ostringstream os;
-    os << "factor: unexpected token type: <" << get_opcode_name(oc) << ">";
-    throw invalid_expression(os.str());
+    return sign_set;
 }
 
 void formula_interpreter::paren()
@@ -667,7 +734,7 @@ void formula_interpreter::paren()
 
     next();
     expression();
-    if (token().get_opcode() != fop_close)
+    if (token_or_throw().get_opcode() != fop_close)
         throw invalid_expression("paren: expected close paren");
 
     if (mp_handler)
@@ -679,17 +746,13 @@ void formula_interpreter::paren()
 void formula_interpreter::single_ref()
 {
     address_t addr = token().get_single_ref();
-#if DEBUG_FORMULA_INTERPRETER
-    __IXION_DEBUG_OUT__ << "formula_interpreter::single_ref: ref=" << addr.get_name() << endl;
-    __IXION_DEBUG_OUT__ << "formula_interpreter::single_ref: origin=" << m_pos.get_name() << endl;
-#endif
+    IXION_TRACE("ref=" << addr.get_name() << "; origin=" << m_pos.get_name());
+
     if (mp_handler)
         mp_handler->push_single_ref(addr, m_pos);
 
     abs_address_t abs_addr = addr.to_abs(m_pos);
-#if DEBUG_FORMULA_INTERPRETER
-    __IXION_DEBUG_OUT__ << "formula_interpreter::single_ref: ref=" << abs_addr.get_name() << " (converted to absolute)" << endl;
-#endif
+    IXION_TRACE("ref=" << abs_addr.get_name() << " (converted to absolute)");
 
     if (abs_addr == m_pos)
     {
@@ -697,25 +760,22 @@ void formula_interpreter::single_ref()
         throw formula_error(formula_error_t::ref_result_not_available);
     }
 
-    m_stack.push_single_ref(abs_addr);
+    get_stack().push_single_ref(abs_addr);
     next();
 }
 
 void formula_interpreter::range_ref()
 {
     range_t range = token().get_range_ref();
-#if DEBUG_FORMULA_INTERPRETER
-    __IXION_DEBUG_OUT__ << "formula_interpreter::range_ref: ref=" << range.first.get_name() << ":" << range.last.get_name() << endl;
-    __IXION_DEBUG_OUT__ << "formula_interpreter::range_ref: origin=" << m_pos.get_name() << endl;
-#endif
+    IXION_TRACE("ref-start=" << range.first.get_name() << "; ref-end=" << range.last.get_name() << "; origin=" << m_pos.get_name());
+
     if (mp_handler)
         mp_handler->push_range_ref(range, m_pos);
 
     abs_range_t abs_range = range.to_abs(m_pos);
+    abs_range.reorder();
 
-#if DEBUG_FORMULA_INTERPRETER
-    __IXION_DEBUG_OUT__ << "formula_interpreter::range_ref: ref=" << abs_range.first.get_name() << ":" << abs_range.last.get_name() << " (converted to absolute)" << endl;
-#endif
+    IXION_TRACE("ref-start=" << abs_range.first.get_name() << "; ref-end=" << abs_range.last.get_name() << " (converted to absolute)");
 
     // Check the reference range to make sure it doesn't include the parent cell.
     if (abs_range.contains(m_pos))
@@ -724,7 +784,7 @@ void formula_interpreter::range_ref()
         throw formula_error(formula_error_t::ref_result_not_available);
     }
 
-    m_stack.push_range_ref(abs_range);
+    get_stack().push_range_ref(abs_range);
     next();
 }
 
@@ -733,9 +793,7 @@ void formula_interpreter::table_ref()
     const iface::table_handler* table_hdl = m_context.get_table_handler();
     if (!table_hdl)
     {
-#if DEBUG_FORMULA_INTERPRETER
-        __IXION_DEBUG_OUT__ << "formula_interpreter::table_ref: failed to get a table_handler instance." << endl;
-#endif
+        IXION_DEBUG("failed to get a table_handler instance.");
         throw formula_error(formula_error_t::ref_result_not_available);
     }
 
@@ -756,7 +814,7 @@ void formula_interpreter::table_ref()
         range = table_hdl->get_range(m_pos, table.column_first, table.column_last, table.areas);
     }
 
-    m_stack.push_range_ref(range);
+    get_stack().push_range_ref(range);
     next();
 }
 
@@ -764,7 +822,7 @@ void formula_interpreter::constant()
 {
     double val = token().get_value();
     next();
-    m_stack.push_value(val);
+    get_stack().push_value(val);
     if (mp_handler)
         mp_handler->push_value(val);
 }
@@ -772,8 +830,12 @@ void formula_interpreter::constant()
 void formula_interpreter::literal()
 {
     size_t sid = token().get_index();
+    const std::string* p = m_context.get_string(sid);
+    if (!p)
+        throw general_error("no string found for the specified string ID.");
+
     next();
-    m_stack.push_string(sid);
+    get_stack().push_string(*p);
     if (mp_handler)
         mp_handler->push_string(sid);
 }
@@ -781,12 +843,17 @@ void formula_interpreter::literal()
 void formula_interpreter::function()
 {
     // <func name> '(' <expression> ',' <expression> ',' ... ',' <expression> ')'
+    ensure_token_exists();
     assert(token().get_opcode() == fop_function);
-    assert(m_stack.empty());
     formula_function_t func_oc = formula_functions::get_function_opcode(token());
     if (mp_handler)
         mp_handler->push_function(func_oc);
 
+    push_stack();
+
+    IXION_TRACE("function='" << get_formula_function_name(func_oc) << "'");
+    assert(get_stack().empty());
+
     if (next_token().get_opcode() != fop_open)
         throw invalid_expression("expecting a '(' after a function name.");
 
@@ -812,7 +879,7 @@ void formula_interpreter::function()
             expression();
             expect_sep = true;
         }
-        oc = token().get_opcode();
+        oc = token_or_throw().get_opcode();
     }
 
     if (mp_handler)
@@ -820,14 +887,40 @@ void formula_interpreter::function()
 
     next();
 
-#if DEBUG_FORMULA_INTERPRETER
-    __IXION_DEBUG_OUT__ << "function: " << formula_functions::get_function_name(func_oc) << endl;
-#endif
     // Function call pops all stack values pushed onto the stack this far, and
     // pushes the result onto the stack.
-    formula_functions(m_context).interpret(func_oc, m_stack);
-    assert(m_stack.size() == 1);
+    formula_functions(m_context).interpret(func_oc, get_stack());
+    assert(get_stack().size() == 1);
+
+    pop_stack();
+}
+
+void formula_interpreter::clear_stacks()
+{
+    m_stacks.clear();
+    m_stacks.emplace_back(m_context);
+}
+
+void formula_interpreter::push_stack()
+{
+    m_stacks.emplace_back(m_context);
+}
+
+void formula_interpreter::pop_stack()
+{
+    assert(m_stacks.size() >= 2);
+    assert(m_stacks.back().size() == 1);
+    auto tmp = m_stacks.back().release_back();
+    m_stacks.pop_back();
+    m_stacks.back().push_back(std::move(tmp));
+}
+
+formula_value_stack& formula_interpreter::get_stack()
+{
+    assert(!m_stacks.empty());
+    return m_stacks.back();
 }
 
 }
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
