@@ -5,8 +5,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#ifndef __IXION_FORMULA_INTERPRETER_HPP__
-#define __IXION_FORMULA_INTERPRETER_HPP__
+#ifndef INCLUDED_IXION_FORMULA_INTERPRETER_HPP
+#define INCLUDED_IXION_FORMULA_INTERPRETER_HPP
 
 #include "ixion/global.hpp"
 #include "ixion/formula_tokens.hpp"
@@ -16,6 +16,7 @@
 
 #include <sstream>
 #include <unordered_set>
+#include <deque>
 
 namespace ixion {
 
@@ -43,7 +44,8 @@ class session_handler;
  */
 class formula_interpreter
 {
-    typedef std::unordered_set< ::std::string> name_set;
+    using name_set = std::unordered_set<std::string>;
+    using fv_stacks_type = std::deque<formula_value_stack>;
 
 public:
     typedef ::std::vector<const formula_token*> local_tokens_type;
@@ -57,7 +59,7 @@ public:
 
     void set_origin(const abs_address_t& pos);
     bool interpret();
-    const formula_result& get_result() const;
+    formula_result transfer_result();
     formula_error_t get_error() const;
 
 private:
@@ -69,12 +71,13 @@ private:
 
     void pop_result();
 
-    void expand_named_expression(
-        const ::std::string& expr_name, const formula_tokens_t* expr, name_set& used_names);
+    void expand_named_expression(const named_expression_t* expr, name_set& used_names);
 
+    void ensure_token_exists() const;
     bool has_token() const;
     void next();
     const formula_token& token() const;
+    const formula_token& token_or_throw() const;
     const formula_token& next_token();
 
     // The following methods are handlers.  In each handler, the initial
@@ -85,6 +88,7 @@ private:
     void expression();
     void term();
     void factor();
+    bool sign();
     void paren();
     void single_ref();
     void range_ref();
@@ -93,13 +97,19 @@ private:
     void literal();
     void function();
 
+    void clear_stacks();
+    void push_stack();
+    void pop_stack();
+
+    formula_value_stack& get_stack();
+
 private:
     const formula_cell* m_parent_cell;
     iface::formula_model_access& m_context;
     std::unique_ptr<iface::session_handler> mp_handler;
     abs_address_t m_pos;
 
-    value_stack_t m_stack;
+    fv_stacks_type m_stacks;
     local_tokens_type m_tokens;
     local_tokens_type::const_iterator m_cur_token_itr;
     local_tokens_type::const_iterator m_end_token_pos;
