@@ -11,6 +11,7 @@
 #include "ixion/table.hpp"
 
 #include "formula_functions.hpp"
+#include "debug.hpp"
 
 #include <cassert>
 #include <iostream>
@@ -19,8 +20,6 @@
 #include <limits>
 #include <algorithm>
 
-#define DEBUG_NAME_RESOLVER 0
-
 using namespace std;
 
 namespace ixion {
@@ -29,12 +28,12 @@ namespace {
 
 bool check_address_by_sheet_bounds(const iface::formula_model_access* cxt, const address_t& pos)
 {
-    sheet_size_t ss(row_upper_bound, column_upper_bound);
+    rc_size_t ss(row_upper_bound, column_upper_bound);
 
     if (cxt && pos.sheet >= 0 && size_t(pos.sheet) < cxt->get_sheet_count())
     {
         // Make sure the address is within the sheet size.
-        ss = cxt->get_sheet_size(pos.sheet);
+        ss = cxt->get_sheet_size();
     }
 
     row_t row_check = pos.row >= 0 ? pos.row : -pos.row;
@@ -261,6 +260,12 @@ enum class resolver_parse_mode { column, row };
 
 void append_sheet_name(ostringstream& os, const ixion::iface::formula_model_access& cxt, sheet_t sheet)
 {
+    if (!is_valid_sheet(sheet))
+    {
+        IXION_DEBUG("invalid sheet index (" << sheet << ")");
+        return;
+    }
+
     string sheet_name = cxt.get_sheet_name(sheet);
     string buffer; // used only when the sheet name contains at least one single quote.
 
@@ -306,6 +311,36 @@ void append_sheet_name(ostringstream& os, const ixion::iface::formula_model_acce
         os << '\'';
 }
 
+void append_sheet_name_calc_a1(
+    std::ostringstream& os, const ixion::iface::formula_model_access* cxt, const address_t& addr, const abs_address_t& origin)
+{
+    if (!cxt)
+        return;
+
+    sheet_t sheet = addr.sheet;
+    if (addr.abs_sheet)
+        os << '$';
+    else
+        sheet += origin.sheet;
+    append_sheet_name(os, *cxt, sheet);
+    os << '.';
+}
+
+void append_sheet_name_odf_cra(
+    std::ostringstream& os, const ixion::iface::formula_model_access* cxt, const address_t& addr, const abs_address_t& origin)
+{
+    if (cxt)
+    {
+        sheet_t sheet = addr.sheet;
+        if (addr.abs_sheet)
+            os << '$';
+        else
+            sheet += origin.sheet;
+        append_sheet_name(os, *cxt, sheet);
+    }
+    os << '.';
+}
+
 void append_column_name_a1(ostringstream& os, col_t col)
 {
     const col_t div = 26;
@@ -327,10 +362,38 @@ void append_column_name_a1(ostringstream& os, col_t col)
     os << col_name;
 }
 
+void append_column_address_a1(std::ostringstream& os, col_t col, col_t origin, bool absolute)
+{
+    if (col == column_unset)
+        return;
+
+    if (absolute)
+        os << '$';
+    else
+        col += origin;
+
+    append_column_name_a1(os, col);
+}
+
+void append_row_address_a1(std::ostringstream& os, row_t row, row_t origin, bool absolute)
+{
+    if (row == row_unset)
+        return;
+
+    if (absolute)
+        os << '$';
+    else
+        row += origin;
+
+    os << (row + 1);
+}
+
 void append_address_a1(
-    ostringstream& os, const ixion::iface::formula_model_access* cxt,
+    std::ostringstream& os, const ixion::iface::formula_model_access* cxt,
     const address_t& addr, const abs_address_t& pos, char sheet_name_sep)
 {
+    assert(sheet_name_sep);
+
     col_t col = addr.column;
     row_t row = addr.row;
     sheet_t sheet = addr.sheet;
@@ -341,7 +404,7 @@ void append_address_a1(
     if (!addr.abs_sheet)
         sheet += pos.sheet;
 
-    if (sheet_name_sep && cxt)
+    if (cxt)
     {
         append_sheet_name(os, *cxt, sheet);
         os << sheet_name_sep;
@@ -356,6 +419,20 @@ void append_address_a1(
     os << (row + 1);
 }
 
+/**
+ * Almost identical to append_address_a1, but it always appends a sheet name
+ * separator even if a sheet name is not appended.
+ */
+void append_address_a1_with_sheet_name_sep(
+    std::ostringstream& os, const ixion::iface::formula_model_access* cxt,
+    const address_t& addr, const abs_address_t& pos, char sheet_name_sep)
+{
+    if (!cxt)
+        os << sheet_name_sep;
+
+    append_address_a1(os, cxt, addr, pos, sheet_name_sep);
+}
+
 void append_address_r1c1(
     ostringstream& os, const address_t& addr, const abs_address_t& pos)
 {
@@ -436,25 +513,45 @@ char append_table_areas(ostringstream& os, const table_t& table)
     return count;
 }
 
-enum parse_address_result
+enum parse_address_result_type
 {
     invalid = 0,
     valid_address,
     range_expected /// valid address followed by a ':'.
 };
 
-#if DEBUG_NAME_RESOLVER
-const char* parse_address_result_names[] = {
-    "invalid",
-    "valid address",
-    "range expected"
+struct parse_address_result
+{
+    parse_address_result_type result;
+    bool sheet_name = false;
 };
+
+#if IXION_LOGGING
+
+std::ostream& operator<< (std::ostream& os, parse_address_result_type rt)
+{
+    static const char* names[] = {
+        "invalid",
+        "valid address",
+        "range expected"
+    };
+
+    os << names[rt];
+    return os;
+}
+
+std::string to_string(parse_address_result_type rt)
+{
+    std::ostringstream os;
+    os << rt;
+    return os.str();
+}
+
 #endif
 
-void parse_sheet_name_quoted(
+bool parse_sheet_name_quoted(
     const ixion::iface::formula_model_access& cxt, const char sep, const char*& p, const char* p_last, sheet_t& sheet)
 {
-    const char* p_old = p;
     ++p; // skip the open quote.
     size_t len = 0;
     string buffer; // used only when the name contains at least one single quote.
@@ -497,7 +594,7 @@ void parse_sheet_name_quoted(
             ++p; // skip the closing quote.
             if (p != p_last)
                 ++p; // skip the separator.
-            return;
+            return true;
         }
 
         if (p == p_last)
@@ -507,18 +604,34 @@ void parse_sheet_name_quoted(
         ++len;
     }
 
-    p = p_old;
+    return false;
 }
 
-void parse_sheet_name(const ixion::iface::formula_model_access& cxt, const char sep, const char*& p, const char* p_last, sheet_t& sheet)
+/**
+ * Try to parse a sheet name prefix in the string.  If this fails, revert
+ * the current position back to the original position prior to the call.
+ *
+ * @return true if the string contains a valid sheet name, false otherwise.
+ */
+bool parse_sheet_name(
+    const ixion::iface::formula_model_access& cxt, const char sep, const char*& p, const char* p_last, sheet_t& sheet)
 {
+    assert(p <= p_last);
+
+    const char* p_old = p; // old position to revert to in case we fail to parse a sheet name.
+
+    if (*p == '$')
+        ++p;
+
     if (*p == '\'')
     {
-        parse_sheet_name_quoted(cxt, sep, p, p_last, sheet);
-        return;
+        bool success = parse_sheet_name_quoted(cxt, sep, p, p_last, sheet);
+        if (!success)
+            p = p_old;
+        return success;
     }
 
-    const char* p_old = p;
+    const char* p0 = p;
     size_t len = 0;
 
     // parse until we hit the sheet-address separator.
@@ -526,10 +639,10 @@ void parse_sheet_name(const ixion::iface::formula_model_access& cxt, const char
     {
         if (*p == sep)
         {
-            sheet = cxt.get_sheet_index(p_old, len);
+            sheet = cxt.get_sheet_index(p0, len);
             if (p != p_last)
                 ++p; // skip the separator.
-            return;
+            return true;
         }
 
         if (p == p_last)
@@ -540,6 +653,7 @@ void parse_sheet_name(const ixion::iface::formula_model_access& cxt, const char
     }
 
     p = p_old;
+    return false;
 }
 
 /**
@@ -595,7 +709,7 @@ T parse_number(const char*&p, const char* p_last)
  *
  * @return parsing result.
  */
-parse_address_result parse_address_a1(const char*& p, const char* p_last, address_t& addr)
+parse_address_result_type parse_address_a1(const char*& p, const char* p_last, address_t& addr)
 {
     // NOTE: Row and column IDs are 1-based during parsing, while 0 is used as
     // the state of a value-not-set.  They are subtracted by one before
@@ -728,7 +842,7 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
     return valid_address;
 }
 
-parse_address_result parse_address_r1c1(const char*& p, const char* p_last, address_t& addr)
+parse_address_result_type parse_address_r1c1(const char*& p, const char* p_last, address_t& addr)
 {
     addr.row = row_unset;
     addr.column = column_unset;
@@ -740,7 +854,7 @@ parse_address_result parse_address_r1c1(const char*& p, const char* p_last, addr
 
         if (p == p_last)
             // Just 'R'.  Not sure if this is valid or invalid, but let's call it invalid for now.
-            return parse_address_result::invalid;
+            return parse_address_result_type::invalid;
 
         ++p;
         if (*p != 'C' && *p != 'c')
@@ -751,12 +865,12 @@ parse_address_result parse_address_r1c1(const char*& p, const char* p_last, addr
                 // Relative row address.
                 ++p;
                 if (!is_digit(*p) && *p != '-' && *p != '+')
-                    return parse_address_result::invalid;
+                    return parse_address_result_type::invalid;
 
                 addr.row = parse_number<row_t>(p, p_last);
                 ++p;
                 if (p == p_last)
-                    return (*p == ']') ? parse_address_result::valid_address : parse_address_result::invalid;
+                    return (*p == ']') ? parse_address_result_type::valid_address : parse_address_result_type::invalid;
                 ++p;
             }
             else if (is_digit(*p))
@@ -765,13 +879,13 @@ parse_address_result parse_address_r1c1(const char*& p, const char* p_last, addr
                 addr.row = parse_number<row_t>(p, p_last);
                 if (addr.row <= 0)
                     // absolute address with 0 or negative value is invalid.
-                    return parse_address_result::invalid;
+                    return parse_address_result_type::invalid;
 
                 --addr.row; // 1-based to 0-based.
 
                 if (p == p_last && is_digit(*p))
                     // 'R' followed by a number without 'C' is valid.
-                    return parse_address_result::valid_address;
+                    return parse_address_result_type::valid_address;
                 ++p;
             }
         }
@@ -786,13 +900,13 @@ parse_address_result parse_address_r1c1(const char*& p, const char* p_last, addr
         {
             if (addr.row == row_unset)
                 // Just 'C'.  Row must be set.
-                return parse_address_result::invalid;
+                return parse_address_result_type::invalid;
 
             if (!addr.abs_row && addr.row == 0)
                 // 'RC' is invalid as it references itself.
-                return parse_address_result::invalid;
+                return parse_address_result_type::invalid;
 
-            return parse_address_result::valid_address;
+            return parse_address_result_type::valid_address;
         }
 
         ++p;
@@ -801,12 +915,12 @@ parse_address_result parse_address_r1c1(const char*& p, const char* p_last, addr
             // Relative column address.
             ++p;
             if (!is_digit(*p) && *p != '-' && *p != '+')
-                return parse_address_result::invalid;
+                return parse_address_result_type::invalid;
 
             addr.column = parse_number<col_t>(p, p_last);
             ++p;
             if (p == p_last)
-                return (*p == ']') ? parse_address_result::valid_address : parse_address_result::invalid;
+                return (*p == ']') ? parse_address_result_type::valid_address : parse_address_result_type::invalid;
 
             ++p;
         }
@@ -817,24 +931,65 @@ parse_address_result parse_address_r1c1(const char*& p, const char* p_last, addr
             addr.column = parse_number<col_t>(p, p_last);
             if (addr.column <= 0)
                 // absolute address with 0 or negative value is invalid.
-                return parse_address_result::invalid;
+                return parse_address_result_type::invalid;
 
             --addr.column; // 1-based to 0-based.
 
             if (p == p_last)
-                return parse_address_result::valid_address;
+                return parse_address_result_type::valid_address;
 
             ++p;
         }
     }
 
     if (*p == ':')
-        return (p == p_last) ? parse_address_result::invalid : parse_address_result::range_expected;
+        return (p == p_last) ? parse_address_result_type::invalid : parse_address_result_type::range_expected;
 
-    return parse_address_result::invalid;
+    return parse_address_result_type::invalid;
 }
 
-parse_address_result parse_address_excel_a1(
+parse_address_result parse_address_calc_a1(
+    const ixion::iface::formula_model_access* cxt, const char*& p, const char* p_last, address_t& addr)
+{
+    parse_address_result res;
+
+    addr.row = 0;
+    addr.column = 0;
+    addr.abs_row = false;
+    addr.abs_column = false;
+
+    if (cxt)
+    {
+        // Overwrite the sheet index *only when* the sheet name is parsed successfully.
+        const char* p0 = p;
+        res.sheet_name = parse_sheet_name(*cxt, '.', p, p_last, addr.sheet);
+        if (res.sheet_name)
+            addr.abs_sheet = (*p0 == '$');
+    }
+
+    res.result = parse_address_a1(p, p_last, addr);
+    return res;
+}
+
+/**
+ * Parse a single cell address in ODF cell-range-address syntax.  This is
+ * almost identical to Calc A1 except that it allows a leading '.' as in
+ * '.E1' as opposed to just 'E1'.
+ */
+parse_address_result parse_address_odf_cra(
+    const ixion::iface::formula_model_access* cxt, const char*& p, const char* p_last, address_t& addr)
+{
+    if (*p == '.')
+    {
+        // Skip the '.', and assume absence of sheet name.
+        ++p;
+        cxt = nullptr;
+    }
+
+    return parse_address_calc_a1(cxt, p, p_last, addr);
+}
+
+parse_address_result_type parse_address_excel_a1(
     const ixion::iface::formula_model_access* cxt, const char*& p, const char* p_last, address_t& addr)
 {
     addr.row = 0;
@@ -850,7 +1005,7 @@ parse_address_result parse_address_excel_a1(
     return parse_address_a1(p, p_last, addr);
 }
 
-parse_address_result parse_address_excel_r1c1(
+parse_address_result_type parse_address_excel_r1c1(
     const iface::formula_model_access* cxt, const char*& p, const char* p_last, address_t& addr)
 {
     addr.row = 0;
@@ -869,24 +1024,39 @@ parse_address_result parse_address_excel_r1c1(
 parse_address_result parse_address_odff(
     const ixion::iface::formula_model_access* cxt, const char*& p, const char* p_last, address_t& addr)
 {
+    parse_address_result res;
+    assert(p <= p_last);
+
     addr.row = 0;
     addr.column = 0;
-    addr.abs_sheet = false;
     addr.abs_row = false;
     addr.abs_column = false;
 
     if (*p == '.')
     {
-        // This address doesn't contain sheet name.
+        // This address doesn't contain a sheet name.
         ++p;
     }
     else if (cxt)
     {
+        // This address DOES contain a sheet name.
+        res.sheet_name = true;
+        addr.abs_sheet = false;
+        addr.sheet = invalid_sheet;
+
         // Overwrite the sheet index *only when* the sheet name is parsed successfully.
-        parse_sheet_name(*cxt, '.', p, p_last, addr.sheet);
+        if (*p == '$')
+        {
+            addr.abs_sheet = true;
+            ++p;
+        }
+
+        if (p <= p_last)
+            parse_sheet_name(*cxt, '.', p, p_last, addr.sheet);
     }
 
-    return parse_address_a1(p, p_last, addr);
+    res.result = parse_address_a1(p, p_last, addr);
+    return res;
 }
 
 string abs_or_rel(bool _abs)
@@ -904,13 +1074,13 @@ string _to_string(const formula_name_t::address_type& addr)
     return os.str();
 }
 
-void to_relative_address(address_t& addr, const abs_address_t& pos)
+void to_relative_address(address_t& addr, const abs_address_t& pos, bool sheet)
 {
-    if (!addr.abs_sheet)
+    if (!addr.abs_sheet && sheet)
         addr.sheet -= pos.sheet;
-    if (!addr.abs_row)
+    if (!addr.abs_row && addr.row <= row_upper_bound)
         addr.row -= pos.row;
-    if (!addr.abs_column)
+    if (!addr.abs_column && addr.column <= column_upper_bound)
         addr.column -= pos.column;
 }
 
@@ -1068,9 +1238,6 @@ public:
 
     virtual formula_name_t resolve(const char* p, size_t n, const abs_address_t& pos) const
     {
-#if DEBUG_NAME_RESOLVER
-        __IXION_DEBUG_OUT__ << "name=" << string(p,n) << "; origin=" << pos.get_name() << endl;
-#endif
         formula_name_t ret;
         if (!n)
             return ret;
@@ -1087,11 +1254,7 @@ public:
         // Use the sheet where the cell is unless sheet name is explicitly given.
         address_t parsed_addr(pos.sheet, 0, 0, false, false, false);
 
-        parse_address_result parse_res = parse_address_excel_a1(mp_cxt, p, p_last, parsed_addr);
-
-#if DEBUG_NAME_RESOLVER
-        __IXION_DEBUG_OUT__ << "parse address result: " << parse_address_result_names[parse_res] << endl;
-#endif
+        parse_address_result_type parse_res = parse_address_excel_a1(mp_cxt, p, p_last, parsed_addr);
 
         if (parse_res != invalid)
         {
@@ -1109,15 +1272,9 @@ public:
         if (parse_res == valid_address && parsed_addr.row != row_unset)
         {
             // This is a single cell address.
-            to_relative_address(parsed_addr, pos);
+            to_relative_address(parsed_addr, pos, true);
             set_cell_reference(ret, parsed_addr);
 
-#if DEBUG_NAME_RESOLVER
-            string abs_row_s = parsed_addr.abs_row ? "abs" : "rel";
-            string abs_col_s = parsed_addr.abs_column ? "abs" : "rel";
-            cout << "resolve: " << string(p,n) << "=(row=" << parsed_addr.row
-                << " [" << abs_row_s << "]; column=" << parsed_addr.column << " [" << abs_col_s << "])" << endl;
-#endif
             return ret;
         }
 
@@ -1129,7 +1286,7 @@ public:
 
             ++p; // skip ':'
 
-            to_relative_address(parsed_addr, pos);
+            to_relative_address(parsed_addr, pos, true);
             set_address(ret.range.first, parsed_addr);
 
             // For now, we assume the sheet index of the end address is identical
@@ -1139,7 +1296,7 @@ public:
                 // The 2nd part after the ':' is not valid.
                 return ret;
 
-            to_relative_address(parsed_addr, pos);
+            to_relative_address(parsed_addr, pos, true);
             set_address(ret.range.last, parsed_addr);
             ret.range.last.sheet = ret.range.first.sheet; // re-use the sheet index of the begin address.
             ret.type = formula_name_t::range_reference;
@@ -1153,8 +1310,7 @@ public:
     virtual string get_name(const address_t& addr, const abs_address_t& pos, bool sheet_name) const
     {
         ostringstream os;
-        char sheet_name_sep = sheet_name ? '!' : 0;
-        append_address_a1(os, mp_cxt, addr, pos, sheet_name_sep);
+        append_address_a1(os, sheet_name ? mp_cxt : nullptr, addr, pos, '!');
         return os.str();
     }
 
@@ -1176,36 +1332,14 @@ public:
             os << '!';
         }
 
-        if (col != column_unset)
-        {
-            if (range.first.abs_column)
-                os << '$';
-            else
-                col += pos.column;
-            append_column_name_a1(os, col);
-        }
-
-        if (row != row_unset)
-        {
-            if (range.first.abs_row)
-                os << '$';
-            else
-                row += pos.row;
-            os << (row + 1);
-        }
+        append_column_address_a1(os, col, pos.column, range.first.abs_column);
+        append_row_address_a1(os, row, pos.row, range.first.abs_row);
 
         os << ":";
         col = range.last.column;
         row = range.last.row;
 
-        if (col != column_unset)
-        {
-            if (range.last.abs_column)
-                os << '$';
-            else
-                col += pos.column;
-            append_column_name_a1(os, col);
-        }
+        append_column_address_a1(os, col, pos.column, range.last.abs_column);
 
         if (row != row_unset)
         {
@@ -1269,7 +1403,7 @@ public:
         // Use the sheet where the cell is unless sheet name is explicitly given.
         address_t parsed_addr(pos.sheet, 0, 0);
 
-        parse_address_result parse_res = parse_address_excel_r1c1(mp_cxt, p, p_last, parsed_addr);
+        parse_address_result_type parse_res = parse_address_excel_r1c1(mp_cxt, p, p_last, parsed_addr);
 
         if (parse_res != invalid)
         {
@@ -1285,20 +1419,20 @@ public:
 
         switch (parse_res)
         {
-            case parse_address_result::valid_address:
+            case parse_address_result_type::valid_address:
             {
                 set_cell_reference(ret, parsed_addr);
                 return ret;
             }
-            case parse_address_result::range_expected:
+            case parse_address_result_type::range_expected:
             {
                 ++p; // skip ':'
                 if (p == p_end)
                     return ret;
 
                 address_t parsed_addr2(0, 0, 0);
-                parse_address_result parse_res2 = parse_address_excel_r1c1(nullptr, p, p_last, parsed_addr2);
-                if (parse_res2 != parse_address_result::valid_address)
+                parse_address_result_type parse_res2 = parse_address_excel_r1c1(nullptr, p, p_last, parsed_addr2);
+                if (parse_res2 != parse_address_result_type::valid_address)
                     return ret;
 
                 // For now, we assume the sheet index of the end address is identical
@@ -1355,6 +1489,173 @@ public:
     }
 };
 
+class dot_a1_resolver : public formula_name_resolver
+{
+    using func_parse_address_type =
+        std::function<parse_address_result(
+            const ixion::iface::formula_model_access*,
+            const char*&, const char*, address_t&)>;
+
+    using func_append_address_type =
+        std::function<void(
+            std::ostringstream&, const ixion::iface::formula_model_access*,
+            const address_t&, const abs_address_t&, char)>;
+
+    using func_append_sheet_name_type =
+        std::function<void(
+            std::ostringstream&, const ixion::iface::formula_model_access*,
+            const address_t&, const abs_address_t&)>;
+
+    const iface::formula_model_access* mp_cxt;
+    func_parse_address_type m_func_parse_address;
+    func_append_address_type m_func_append_address;
+    func_append_sheet_name_type m_func_append_sheet_name;
+
+    static bool display_last_sheet(const range_t& range, const abs_address_t& pos)
+    {
+        if (range.first.abs_sheet != range.last.abs_sheet)
+            return true;
+
+        abs_range_t abs = range.to_abs(pos);
+        return abs.first.sheet != abs.last.sheet;
+    }
+
+public:
+    dot_a1_resolver(
+        const iface::formula_model_access* cxt,
+        func_parse_address_type func_parse_address,
+        func_append_address_type func_append_address,
+        func_append_sheet_name_type func_append_sheet_name) :
+        formula_name_resolver(),
+        mp_cxt(cxt),
+        m_func_parse_address(func_parse_address),
+        m_func_append_address(func_append_address),
+        m_func_append_sheet_name(func_append_sheet_name) {}
+
+    virtual formula_name_t resolve(const char *p, size_t n, const abs_address_t &pos) const override
+    {
+        formula_name_t ret;
+        if (!n)
+            return ret;
+
+        if (resolve_function(p, n, ret))
+            return ret;
+
+        const char* p_last = p + n -1;
+
+        // Use the sheet where the cell is unless sheet name is explicitly given.
+        address_t parsed_addr(pos.sheet, 0, 0, false, false, false);
+
+        parse_address_result parse_res = m_func_parse_address(mp_cxt, p, p_last, parsed_addr);
+        IXION_TRACE("parse address result on 1st address (" << to_string(parse_res.result) << ")");
+
+        if (parse_res.result != invalid)
+        {
+            // This is a valid A1-style address syntax-wise.
+
+            if (parsed_addr.sheet == invalid_sheet)
+            {
+                IXION_DEBUG("Sheet name is not found in the model context.");
+                return ret;
+            }
+
+            if (!check_address_by_sheet_bounds(mp_cxt, parsed_addr))
+            {
+                IXION_DEBUG("Address is outside the sheet bounds.");
+                parse_res.result = invalid;
+            }
+        }
+
+        // prevent for example H to be recognized as column address
+        if (parse_res.result == valid_address && parsed_addr.row != row_unset)
+        {
+            // This is a single cell address.
+            to_relative_address(parsed_addr, pos, true);
+            set_cell_reference(ret, parsed_addr);
+
+            return ret;
+        }
+
+        if (parse_res.result == range_expected)
+        {
+            if (p == p_last)
+            {
+                IXION_DEBUG("':' occurs as the last character.  This is not allowed.");
+                return ret;
+            }
+
+            ++p; // skip ':'
+
+            to_relative_address(parsed_addr, pos, true);
+            set_address(ret.range.first, parsed_addr);
+
+            parse_res = m_func_parse_address(mp_cxt, p, p_last, parsed_addr);
+            IXION_TRACE("parse address result on 2nd address (" << to_string(parse_res.result) << ")");
+
+            if (parse_res.result != valid_address)
+            {
+                IXION_DEBUG("2nd part after the ':' is not valid.");
+                return ret;
+            }
+
+            to_relative_address(parsed_addr, pos, parse_res.sheet_name);
+            set_address(ret.range.last, parsed_addr);
+            ret.type = formula_name_t::range_reference;
+            return ret;
+        }
+
+        resolve_function_or_name(p, n, ret);
+        return ret;
+    }
+
+    virtual std::string get_name(const address_t &addr, const abs_address_t &pos, bool sheet_name) const override
+    {
+        std::ostringstream os;
+        if (sheet_name && addr.abs_sheet)
+            os << '$';
+        m_func_append_address(os, sheet_name ? mp_cxt : nullptr, addr, pos, '.');
+        return os.str();
+    }
+
+    virtual std::string get_name(const range_t& range, const abs_address_t& pos, bool sheet_name) const override
+    {
+        ostringstream os;
+        col_t col = range.first.column;
+        row_t row = range.first.row;
+
+        m_func_append_sheet_name(os, sheet_name ? mp_cxt : nullptr, range.first, pos);
+
+        append_column_address_a1(os, col, pos.column, range.first.abs_column);
+        append_row_address_a1(os, row, pos.row, range.first.abs_row);
+
+        os << ":";
+
+        if (!display_last_sheet(range, pos))
+            sheet_name = false;
+
+        m_func_append_sheet_name(os, sheet_name ? mp_cxt : nullptr, range.last, pos);
+
+        col = range.last.column;
+        row = range.last.row;
+
+        append_column_address_a1(os, col, pos.column, range.last.abs_column);
+        append_row_address_a1(os, row, pos.row, range.last.abs_row);
+
+        return os.str();
+    }
+
+    virtual std::string get_name(const table_t &table) const override
+    {
+        // TODO : find out how Calc A1 handles table reference.
+        return std::string();
+    }
+
+    virtual std::string get_column_name(col_t col) const override
+    {
+        return get_column_name_a1(col);
+    }
+};
+
 /**
  * Name resolver for ODFF formula expressions.
  */
@@ -1375,9 +1676,12 @@ public:
             // Empty string.
             return ret;
 
-        // First character must be '['.
+        // First character must be '[' for this to be a reference.
         if (*p != '[')
+        {
+            ret.type = formula_name_t::named_expression;
             return ret;
+        }
 
         ++p;
         const char* p_last = p;
@@ -1390,20 +1694,20 @@ public:
         --p_last;
 
         // Use the sheet where the cell is unless sheet name is explicitly given.
-        address_t parsed_addr(pos.sheet, 0, 0, false, false, false);
+        address_t parsed_addr(pos.sheet, 0, 0, true, false, false);
 
         parse_address_result parse_res = parse_address_odff(mp_cxt, p, p_last, parsed_addr);
 
         // prevent for example H to be recognized as column address
-        if (parse_res == valid_address && parsed_addr.row != row_unset)
+        if (parse_res.result == valid_address && parsed_addr.row != row_unset)
         {
             // This is a single cell address.
-            to_relative_address(parsed_addr, pos);
+            to_relative_address(parsed_addr, pos, true);
             set_cell_reference(ret, parsed_addr);
             return ret;
         }
 
-        if (parse_res == range_expected)
+        if (parse_res.result == range_expected)
         {
             if (p == p_last)
                 // ':' occurs as the last character.  This is not allowed.
@@ -1411,19 +1715,17 @@ public:
 
             ++p; // skip ':'
 
-            to_relative_address(parsed_addr, pos);
+            to_relative_address(parsed_addr, pos, true);
             set_address(ret.range.first, parsed_addr);
 
-            // For now, we assume the sheet index of the end address is identical
-            // to that of the begin address.
-            parse_res = parse_address_odff(NULL, p, p_last, parsed_addr);
-            if (parse_res != valid_address)
+            parse_res = parse_address_odff(mp_cxt, p, p_last, parsed_addr);
+            if (parse_res.result != valid_address)
                 // The 2nd part after the ':' is not valid.
                 return ret;
 
-            to_relative_address(parsed_addr, pos);
+            to_relative_address(parsed_addr, pos, parse_res.sheet_name);
+
             set_address(ret.range.last, parsed_addr);
-            ret.range.last.sheet = ret.range.first.sheet; // re-use the sheet index of the begin address.
             ret.type = formula_name_t::range_reference;
             return ret;
         }
@@ -1435,14 +1737,21 @@ public:
 
     virtual string get_name(const address_t& addr, const abs_address_t& pos, bool sheet_name) const
     {
+        if (!mp_cxt)
+            sheet_name = false;
+
         ostringstream os;
         os << '[';
         if (sheet_name)
+        {
+            if (addr.abs_sheet)
+                os << '$';
             append_address_a1(os, mp_cxt, addr, pos, '.');
+        }
         else
         {
             os << '.';
-            append_address_a1(os, NULL, addr, pos, 0);
+            append_address_a1(os, nullptr, addr, pos, '.');
         }
         os << ']';
         return os.str();
@@ -1450,21 +1759,43 @@ public:
 
     virtual string get_name(const range_t& range, const abs_address_t& pos, bool sheet_name) const
     {
+        if (!mp_cxt)
+            sheet_name = false;
+
         ostringstream os;
         os << '[';
 
         if (sheet_name)
         {
-            append_address_a1(os, mp_cxt, range.first, pos, '.');
+            const iface::formula_model_access* cxt = mp_cxt;
+
+            if (range.first.abs_sheet)
+                os << '$';
+            append_address_a1(os, cxt, range.first, pos, '.');
+
             os << ':';
-            append_address_a1(os, mp_cxt, range.last, pos, '.');
+
+            if (range.last.sheet != range.first.sheet || range.last.abs_sheet != range.first.abs_sheet)
+            {
+                // sheet indices differ between the first and last addresses.
+                if (range.last.abs_sheet)
+                    os << '$';
+            }
+            else
+            {
+                // sheet indices are identical.
+                cxt = nullptr;
+                os << '.';
+            }
+
+            append_address_a1(os, cxt, range.last, pos, '.');
         }
         else
         {
             os << '.';
-            append_address_a1(os, NULL, range.first, pos, 0);
+            append_address_a1(os, nullptr, range.first, pos, '.');
             os << ":.";
-            append_address_a1(os, NULL, range.last, pos, 0);
+            append_address_a1(os, nullptr, range.last, pos, '.');
         }
 
         os << ']';
@@ -1501,6 +1832,13 @@ std::unique_ptr<formula_name_resolver> formula_name_resolver::get(
         case formula_name_resolver_t::odff:
             return std::unique_ptr<formula_name_resolver>(new odff_resolver(cxt));
         case formula_name_resolver_t::calc_a1:
+            return std::unique_ptr<formula_name_resolver>(
+                new dot_a1_resolver(
+                    cxt, parse_address_calc_a1, append_address_a1, append_sheet_name_calc_a1));
+        case formula_name_resolver_t::odf_cra:
+            return std::unique_ptr<formula_name_resolver>(
+                new dot_a1_resolver(
+                    cxt, parse_address_odf_cra, append_address_a1_with_sheet_name_sep, append_sheet_name_odf_cra));
         case formula_name_resolver_t::unknown:
         default:
             ;
