@@ -58,46 +58,43 @@ void formula_parser::set_origin(const abs_address_t& pos)
 
 void formula_parser::parse()
 {
-    try
+    for (m_itr_cur = m_tokens.begin(); has_token(); next())
     {
-        m_itr_cur = m_tokens.begin();
-        for (m_itr_cur = m_tokens.begin(); has_token(); next())
+        const lexer_token_base& t = get_token();
+        lexer_opcode_t oc = t.get_opcode();
+        switch (oc)
         {
-            const lexer_token_base& t = get_token();
-            lexer_opcode_t oc = t.get_opcode();
-            switch (oc)
-            {
-                case lexer_opcode_t::open:
-                case lexer_opcode_t::close:
-                case lexer_opcode_t::plus:
-                case lexer_opcode_t::minus:
-                case lexer_opcode_t::multiply:
-                case lexer_opcode_t::equal:
-                case lexer_opcode_t::divide:
-                case lexer_opcode_t::sep:
-                    primitive(oc);
-                    break;
-                case lexer_opcode_t::name:
-                    name(t);
-                    break;
-                case lexer_opcode_t::string:
-                    literal(t);
-                    break;
-                case lexer_opcode_t::value:
-                    value(t);
-                    break;
-                case lexer_opcode_t::less:
-                    less(t);
-                    break;
-                case lexer_opcode_t::greater:
-                    greater(t);
-                    break;
-                default:
-                    ;
-            }
+            case lexer_opcode_t::open:
+            case lexer_opcode_t::close:
+            case lexer_opcode_t::plus:
+            case lexer_opcode_t::minus:
+            case lexer_opcode_t::multiply:
+            case lexer_opcode_t::exponent:
+            case lexer_opcode_t::concat:
+            case lexer_opcode_t::equal:
+            case lexer_opcode_t::divide:
+            case lexer_opcode_t::sep:
+                primitive(oc);
+                break;
+            case lexer_opcode_t::name:
+                name(t);
+                break;
+            case lexer_opcode_t::string:
+                literal(t);
+                break;
+            case lexer_opcode_t::value:
+                value(t);
+                break;
+            case lexer_opcode_t::less:
+                less(t);
+                break;
+            case lexer_opcode_t::greater:
+                greater(t);
+                break;
+            default:
+                ;
         }
     }
-    catch (const general_error&) {}
 }
 
 void formula_parser::print_tokens() const
@@ -126,6 +123,12 @@ void formula_parser::primitive(lexer_opcode_t oc)
         case lexer_opcode_t::multiply:
             foc = fop_multiply;
             break;
+        case lexer_opcode_t::exponent:
+            foc = fop_exponent;
+            break;
+        case lexer_opcode_t::concat:
+            foc = fop_concat;
+            break;
         case lexer_opcode_t::equal:
             foc = fop_equal;
             break;
@@ -188,8 +191,8 @@ void formula_parser::name(const lexer_token_base& t)
         break;
         default:
         {
-            ostringstream os;
-            os << "failed to resolve a name '" << name.str() << "'.";
+            std::ostringstream os;
+            os << "failed to resolve a name token '" << name.str() << "'.";
             throw parse_error(os.str());
         }
     }
