@@ -9,9 +9,13 @@
 #include "ixion/mem_str_buf.hpp"
 #include "ixion/exceptions.hpp"
 #include "ixion/interface/formula_model_access.hpp"
+#include "ixion/config.hpp"
+#include "ixion/matrix.hpp"
 
 #include <cassert>
 #include <sstream>
+#include <iomanip>
+#include <ostream>
 
 #define DEBUG_FORMULA_RESULT 0
 
@@ -26,16 +30,20 @@ namespace ixion {
 struct formula_result::impl
 {
     result_type m_type;
-    union {
-        string_id_t m_str_identifier;
+
+    union
+    {
         double m_value;
         formula_error_t m_error;
+        matrix* m_matrix;
+        std::string* m_str;
     };
 
     impl() : m_type(result_type::value), m_value(0.0) {}
     impl(double v) : m_type(result_type::value), m_value(v) {}
-    impl(string_id_t strid) : m_type(result_type::string), m_str_identifier(strid) {}
+    impl(std::string str) : m_type(result_type::string), m_str(new std::string(std::move(str))) {}
     impl(formula_error_t e) : m_type(result_type::error), m_error(e) {}
+    impl(matrix mtx) : m_type(result_type::matrix), m_matrix(new matrix(std::move(mtx))) {}
 
     impl(const impl& other) : m_type(other.m_type)
     {
@@ -43,52 +51,98 @@ struct formula_result::impl
         {
             case result_type::value:
                 m_value = other.m_value;
-            break;
+                break;
             case result_type::string:
-                m_str_identifier = other.m_str_identifier;
-            break;
+                m_str = new std::string(*other.m_str);
+                break;
             case result_type::error:
                 m_error = other.m_error;
-            break;
+                break;
+            case result_type::matrix:
+                m_matrix = new matrix(*other.m_matrix);
+                break;
             default:
                 assert(!"unknown formula result type specified during copy construction.");
         }
     }
 
+    ~impl()
+    {
+        delete_buffer();
+    }
+
+    void delete_buffer()
+    {
+        switch (m_type)
+        {
+            case result_type::matrix:
+                delete m_matrix;
+                break;
+            case result_type::string:
+                delete m_str;
+                break;
+            default:
+                ;
+        }
+    }
+
     void reset()
     {
+        delete_buffer();
         m_type = result_type::value;
         m_value = 0.0;
     }
 
     void set_value(double v)
     {
+        delete_buffer();
         m_type = result_type::value;
         m_value = v;
     }
 
-    void set_string(string_id_t strid)
+    void set_string_value(std::string str)
     {
+        if (m_type == result_type::string)
+        {
+            *m_str = std::move(str);
+            return;
+        }
+
+        delete_buffer();
         m_type = result_type::string;
-        m_str_identifier = strid;
+        m_str = new std::string(std::move(str));
     }
 
     void set_error(formula_error_t e)
     {
+        delete_buffer();
         m_type = result_type::error;
         m_error = e;
     }
 
+    void set_matrix(matrix mtx)
+    {
+        if (m_type == result_type::matrix)
+        {
+            *m_matrix = std::move(mtx);
+            return;
+        }
+
+        delete_buffer();
+        m_type = result_type::matrix;
+        m_matrix = new matrix(std::move(mtx));
+    }
+
     double get_value() const
     {
         assert(m_type == result_type::value);
         return m_value;
     }
 
-    string_id_t get_string() const
+    const std::string& get_string_value() const
     {
         assert(m_type == result_type::string);
-        return m_str_identifier;
+        return *m_str;
     }
 
     formula_error_t get_error() const
@@ -97,6 +151,18 @@ struct formula_result::impl
         return m_error;
     }
 
+    const matrix& get_matrix() const
+    {
+        assert(m_type == result_type::matrix);
+        return *m_matrix;
+    }
+
+    matrix& get_matrix()
+    {
+        assert(m_type == result_type::matrix);
+        return *m_matrix;
+    }
+
     result_type get_type() const
     {
         return m_type;
@@ -109,16 +175,67 @@ struct formula_result::impl
             case result_type::error:
                 return string(get_formula_error_name(m_error));
             case result_type::string:
-            {
-                const string* p = cxt.get_string(m_str_identifier);
-                return p ? *p : string();
-            }
+                return *m_str;
             case result_type::value:
             {
                 ostringstream os;
+                if (cxt.get_config().output_precision >= 0)
+                    os << std::fixed << std::setprecision(cxt.get_config().output_precision);
                 os << m_value;
                 return os.str();
             }
+            case result_type::matrix:
+            {
+                const matrix& m = *m_matrix;
+
+                std::ostringstream os;
+
+                os << '{';
+
+                for (size_t row = 0; row < m.row_size(); ++row)
+                {
+                    if (row > 0)
+                        os << cxt.get_config().sep_matrix_row;
+
+                    for (size_t col = 0; col < m.col_size(); ++col)
+                    {
+                        if (col > 0)
+                            os << cxt.get_config().sep_matrix_column;
+
+                        matrix::element e = m.get(row, col);
+
+                        switch (e.type)
+                        {
+                            case matrix::element_type::numeric:
+                            {
+                                os << e.numeric;
+                                break;
+                            }
+                            case matrix::element_type::string:
+                            {
+                                os << '"' << *e.str << '"';
+                                break;
+                            }
+                            case matrix::element_type::error:
+                            {
+                                os << get_formula_error_name(e.error);
+                                break;
+                            }
+                            case matrix::element_type::boolean:
+                            {
+                                os << e.boolean;
+                                break;
+                            }
+                            default:
+                                ;
+                        }
+                    }
+                }
+
+                os << '}';
+
+                return os.str();
+            }
             default:
                 assert(!"unknown formula result type!");
         }
@@ -146,6 +263,7 @@ struct formula_result::impl
             case 'f':
             {
                 // parse this as a boolean value.
+                delete_buffer();
                 m_value = global::to_bool(p, n) ? 1.0 : 0.0;
                 m_type = result_type::value;
                 break;
@@ -153,26 +271,34 @@ struct formula_result::impl
             default:
             {
                 // parse this as a number.
+                delete_buffer();
                 m_value = global::to_double(p, n);
                 m_type = result_type::value;
             }
         }
     }
 
-    void assign_from(const formula_result& r)
+    void move_from(formula_result&& r)
     {
+        delete_buffer();
         m_type = r.mp_impl->m_type;
+
         switch (m_type)
         {
             case result_type::value:
                 m_value = r.mp_impl->m_value;
-            break;
+                break;
             case result_type::string:
-                m_str_identifier = r.mp_impl->m_str_identifier;
-            break;
+                m_str = r.mp_impl->m_str;
+                r.mp_impl->m_str = nullptr;
+                break;
             case result_type::error:
                 m_error = r.mp_impl->m_error;
-            break;
+                break;
+            case result_type::matrix:
+                m_matrix = r.mp_impl->m_matrix;
+                r.mp_impl->m_matrix = nullptr;
+                break;
             default:
                 assert(!"unknown formula result type specified during copy construction.");
         }
@@ -187,13 +313,12 @@ struct formula_result::impl
         {
             case result_type::value:
                 return m_value == r.mp_impl->m_value;
-            break;
             case result_type::string:
-                return m_str_identifier == r.mp_impl->m_str_identifier;
-            break;
+                return *m_str == *r.mp_impl->m_str;
             case result_type::error:
                 return m_error == r.mp_impl->m_error;
-            break;
+            case result_type::matrix:
+                return *m_matrix == *r.mp_impl->m_matrix;
             default:
                 assert(!"unknown formula result type specified during copy construction.");
         }
@@ -228,10 +353,12 @@ struct formula_result::impl
 
                     if (buf.equals("REF"))
                     {
+                        delete_buffer();
                         m_error = formula_error_t::ref_result_not_available;
                     }
                     else if (buf.equals("DIV/0"))
                     {
+                        delete_buffer();
                         m_error = formula_error_t::division_by_zero;
                     }
                     else
@@ -252,7 +379,10 @@ struct formula_result::impl
                     }
 
                     if (buf.equals("NAME"))
+                    {
+                        delete_buffer();
                         m_error = formula_error_t::name_not_found;
+                    }
                     else
                     {
                         good = false;
@@ -298,8 +428,9 @@ struct formula_result::impl
         if (!len)
             throw general_error("failed to parse string result.");
 
+        delete_buffer();
         m_type = result_type::string;
-        m_str_identifier = cxt.add_string(p_first, len);
+        m_str = new std::string(p_first, len);
     }
 };
 
@@ -309,12 +440,16 @@ formula_result::formula_result() :
 formula_result::formula_result(const formula_result& r) :
     mp_impl(ixion::make_unique<impl>(*r.mp_impl)) {}
 
+formula_result::formula_result(formula_result&& r) : mp_impl(std::move(r.mp_impl)) {}
+
 formula_result::formula_result(double v) : mp_impl(ixion::make_unique<impl>(v)) {}
 
-formula_result::formula_result(string_id_t strid) : mp_impl(ixion::make_unique<impl>(strid)) {}
+formula_result::formula_result(std::string str) : mp_impl(ixion::make_unique<impl>(std::move(str))) {}
 
 formula_result::formula_result(formula_error_t e) : mp_impl(ixion::make_unique<impl>(e)) {}
 
+formula_result::formula_result(matrix mtx) : mp_impl(ixion::make_unique<impl>(std::move(mtx))) {}
+
 formula_result::~formula_result() {}
 
 void formula_result::reset()
@@ -327,9 +462,9 @@ void formula_result::set_value(double v)
     mp_impl->set_value(v);
 }
 
-void formula_result::set_string(string_id_t strid)
+void formula_result::set_string_value(std::string str)
 {
-    mp_impl->set_string(strid);
+    mp_impl->set_string_value(std::move(str));
 }
 
 void formula_result::set_error(formula_error_t e)
@@ -337,14 +472,19 @@ void formula_result::set_error(formula_error_t e)
     mp_impl->set_error(e);
 }
 
+void formula_result::set_matrix(matrix mtx)
+{
+    mp_impl->set_matrix(std::move(mtx));
+}
+
 double formula_result::get_value() const
 {
     return mp_impl->get_value();
 }
 
-string_id_t formula_result::get_string() const
+const std::string& formula_result::get_string() const
 {
-    return mp_impl->get_string();
+    return mp_impl->get_string_value();
 }
 
 formula_error_t formula_result::get_error() const
@@ -352,6 +492,16 @@ formula_error_t formula_result::get_error() const
     return mp_impl->get_error();
 }
 
+const matrix& formula_result::get_matrix() const
+{
+    return mp_impl->get_matrix();
+}
+
+matrix& formula_result::get_matrix()
+{
+    return mp_impl->get_matrix();
+}
+
 formula_result::result_type formula_result::get_type() const
 {
     return mp_impl->get_type();
@@ -367,9 +517,9 @@ void formula_result::parse(iface::formula_model_access& cxt, const char* p, size
     mp_impl->parse(cxt, p, n);
 }
 
-formula_result& formula_result::operator= (const formula_result& r)
+formula_result& formula_result::operator= (formula_result r)
 {
-    mp_impl->assign_from(r);
+    mp_impl->move_from(std::move(r));
     return *this;
 }
 
@@ -383,5 +533,30 @@ bool formula_result::operator!= (const formula_result& r) const
     return !operator== (r);
 }
 
+std::ostream& operator<< (std::ostream& os, formula_result::result_type v)
+{
+
+    switch (v)
+    {
+        case formula_result::result_type::error:
+            os << "error";
+            break;
+        case formula_result::result_type::matrix:
+            os << "matrix";
+            break;
+        case formula_result::result_type::string:
+            os << "string";
+            break;
+        case formula_result::result_type::value:
+            os << "value";
+            break;
+        default:
+            ;
+    }
+
+    return os;
 }
+
+}
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
