@@ -24,11 +24,11 @@ public:
     worksheet(size_type row_size, size_type col_size);
     ~worksheet();
 
-    column_store_t& operator[](size_type n) { return *m_columns[n]; }
-    const column_store_t& operator[](size_type n) const { return *m_columns[n]; }
+    column_store_t& operator[](size_type n) { return m_columns[n]; }
+    const column_store_t& operator[](size_type n) const { return m_columns[n]; }
 
-    column_store_t& at(size_type n) { return *m_columns.at(n); }
-    const column_store_t& at(size_type n) const { return *m_columns.at(n); }
+    column_store_t& at(size_type n) { return m_columns.at(n); }
+    const column_store_t& at(size_type n) const { return m_columns.at(n); }
 
     column_store_t::iterator& get_pos_hint(size_type n) { return m_pos_hints.at(n); }
 
@@ -44,8 +44,6 @@ public:
     detail::named_expressions_t& get_named_expressions() { return m_named_expressions; }
     const detail::named_expressions_t& get_named_expressions() const { return m_named_expressions; }
 
-    rc_size_t get_sheet_size() const;
-
 private:
     column_stores_t m_columns;
     std::vector<column_store_t::iterator> m_pos_hints;
@@ -59,11 +57,11 @@ public:
     workbook(size_t sheet_size, size_t row_size, size_t col_size);
     ~workbook();
 
-    worksheet& operator[](size_t n) { return *m_sheets[n]; }
-    const worksheet& operator[](size_t n) const { return *m_sheets[n]; }
+    worksheet& operator[](size_t n) { return m_sheets[n]; }
+    const worksheet& operator[](size_t n) const { return m_sheets[n]; }
 
-    worksheet& at(size_t n) { return *m_sheets.at(n); }
-    const worksheet& at(size_t n) const { return *m_sheets.at(n); }
+    worksheet& at(size_t n) { return m_sheets.at(n); }
+    const worksheet& at(size_t n) const { return m_sheets.at(n); }
 
     void push_back(size_t row_size, size_t col_size);
 
@@ -71,7 +69,7 @@ public:
     bool empty() const;
 
 private:
-    std::vector<worksheet*> m_sheets;
+    std::deque<worksheet> m_sheets;
 };
 
 }
