@@ -12,6 +12,9 @@
 #include "ixion/formula_result.hpp"
 #include "ixion/macros.hpp"
 #include "ixion/address_iterator.hpp"
+#include "ixion/dirty_cell_tracker.hpp"
+#include "ixion/cell_access.hpp"
+#include "ixion/config.hpp"
 
 #include <sstream>
 #include <iostream>
@@ -21,6 +24,8 @@
 #include <cassert>
 #include <memory>
 
+#include <mdds/sorted_string_map.hpp>
+
 using namespace std;
 
 #define DEBUG_MODEL_PARSER 0
@@ -105,9 +110,62 @@ public:
     }
 };
 
+void print_section_title(const char* title)
+{
+    std::cout << get_formula_result_output_separator() << std::endl << title << std::endl;
 }
 
-// ============================================================================
+namespace commands {
+
+enum class type
+{
+    unknown,
+    comment,
+    calc,
+    recalc,
+    check,
+    exit,
+    push,
+    mode_init,
+    mode_edit,
+    mode_result,
+    mode_result_cache,
+    mode_table,
+    mode_session,
+    mode_named_expression,
+    print_dependency,
+};
+
+typedef mdds::sorted_string_map<type> map_type;
+
+// Keys must be sorted.
+const std::vector<map_type::entry> entries =
+{
+    { IXION_ASCII("%"),                     type::comment               },
+    { IXION_ASCII("calc"),                  type::calc                  },
+    { IXION_ASCII("check"),                 type::check                 },
+    { IXION_ASCII("exit"),                  type::exit                  },
+    { IXION_ASCII("mode edit"),             type::mode_edit             },
+    { IXION_ASCII("mode init"),             type::mode_init             },
+    { IXION_ASCII("mode named-expression"), type::mode_named_expression },
+    { IXION_ASCII("mode result"),           type::mode_result           },
+    { IXION_ASCII("mode result-cache"),     type::mode_result_cache     },
+    { IXION_ASCII("mode session"),          type::mode_session          },
+    { IXION_ASCII("mode table"),            type::mode_table            },
+    { IXION_ASCII("print dependency"),      type::print_dependency      },
+    { IXION_ASCII("push"),                  type::push                  },
+    { IXION_ASCII("recalc"),                type::recalc                },
+};
+
+const map_type& get()
+{
+    static map_type mt(entries.data(), entries.size(), type::unknown);
+    return mt;
+}
+
+} // namespace commands
+
+} // anonymous namespace
 
 model_parser::parse_error::parse_error(const string& msg) : general_error()
 {
@@ -124,7 +182,7 @@ model_parser::check_error::check_error(const string& msg) :
 // ============================================================================
 
 model_parser::model_parser(const string& filepath, size_t thread_count) :
-    m_context(),
+    m_context({1048576, 1024}),
     m_table_handler(),
     m_session_handler_factory(m_context),
     mp_table_entry(nullptr),
@@ -134,8 +192,6 @@ model_parser::model_parser(const string& filepath, size_t thread_count) :
     mp_head(nullptr),
     mp_end(nullptr),
     mp_char(nullptr),
-    m_row_limit(1048576),
-    m_col_limit(1024),
     m_current_sheet(0),
     m_parse_mode(parse_mode_unknown),
     m_print_separator(false),
@@ -186,6 +242,9 @@ void model_parser::parse()
             case parse_mode_result:
                 parse_result();
                 break;
+            case parse_mode_result_cache:
+                parse_result_cache();
+                break;
             case parse_mode_table:
                 parse_table();
                 break;
@@ -204,108 +263,143 @@ void model_parser::parse()
 void model_parser::init_model()
 {
     if (m_context.empty())
-        m_context.append_sheet(IXION_ASCII("sheet"), m_row_limit, m_col_limit);
+        m_context.append_sheet(IXION_ASCII("sheet"));
 }
 
 void model_parser::parse_command()
 {
     // This line contains a command.
-    mem_str_buf buf_com = parse_command_to_buffer(mp_char, mp_end);
+    mem_str_buf buf_cmd = parse_command_to_buffer(mp_char, mp_end);
+    commands::type cmd = commands::get().find(buf_cmd.get(), buf_cmd.size());
 
-    if (buf_com.equals("%"))
+    switch (cmd)
     {
-        // This is a comment line.  Just ignore it.
-    }
-    else if (buf_com.equals("calc"))
-    {
-        // Perform full calculation on currently stored cells.
-
-        for (const abs_address_t& pos : m_dirty_cells)
-            register_formula_cell(m_context, pos);
-
-        calculate_cells(m_context, m_dirty_cells, m_thread_count);
-    }
-    else if (buf_com.equals("recalc"))
-    {
-        cout << get_formula_result_output_separator() << endl
-            << "recalculating" << endl;
-
-        get_all_dirty_cells(m_context, m_dirty_cell_addrs, m_dirty_cells);
-        calculate_cells(m_context, m_dirty_cells, m_thread_count);
-    }
-    else if (buf_com.equals("check"))
-    {
-        // Check cell results.
-        check();
-    }
-    else if (buf_com.equals("exit"))
-    {
-        // Exit the loop.
-        m_parse_mode = parse_mode_exit;
-        return;
-    }
-    else if (buf_com.equals("push"))
-    {
-        switch (m_parse_mode)
+        case commands::type::comment:
+            // This is a comment line.  Just ignore it.
+            break;
+        case commands::type::calc:
         {
-            case parse_mode_table:
-                push_table();
-                break;
-            case parse_mode_named_expression:
-                push_named_expression();
-                break;
-            default:
-                throw parse_error("push command was used for wrong mode!");
+            print_section_title("calculating");
+
+            // Perform full calculation on all currently stored formula cells.
+
+            for (const abs_range_t& pos : m_dirty_formula_cells)
+                register_formula_cell(m_context, pos.first);
+
+            abs_range_set_t empty;
+            std::vector<abs_range_t> sorted_cells =
+                query_and_sort_dirty_cells(m_context, empty, &m_dirty_formula_cells);
+            calculate_sorted_cells(m_context, sorted_cells, m_thread_count);
+            break;
         }
-    }
-    else if (buf_com.equals("mode init"))
-    {
-        cout << get_formula_result_output_separator() << endl
-            << "initializing" << endl;
+        case commands::type::recalc:
+        {
+            print_section_title("recalculating");
 
-        m_parse_mode = parse_mode_init;
-        m_print_separator = true;
-    }
-    else if (buf_com.equals("mode result"))
-    {
-        // Clear any previous result values.
-        m_formula_results.clear();
-        m_parse_mode = parse_mode_result;
-    }
-    else if (buf_com.equals("mode edit"))
-    {
-        cout << get_formula_result_output_separator() << endl
-            << "editing" << endl;
+            // Perform partial recalculation only on those formula cells that
+            // need recalculation.
 
-        m_parse_mode = parse_mode_edit;
-        m_dirty_cells.clear();
-        m_dirty_cell_addrs.clear();
-        m_print_separator = true;
-    }
-    else if (buf_com.equals("mode table"))
-    {
-        m_parse_mode = parse_mode_table;
-        mp_table_entry.reset(new table_handler::entry);
-    }
-    else if (buf_com.equals("mode session"))
-    {
-        cout << get_formula_result_output_separator() << endl
-            << "session" << endl;
+            std::vector<abs_range_t> sorted_cells =
+                query_and_sort_dirty_cells(m_context, m_modified_cells, &m_dirty_formula_cells);
 
-        m_print_separator = true;
-        m_parse_mode = parse_mode_session;
-    }
-    else if (buf_com.equals("mode named-expression"))
-    {
-        m_print_separator = true;
-        m_parse_mode = parse_mode_named_expression;
-        mp_named_expression = ixion::make_unique<named_expression_type>();
-    }
-    else
-    {
-        ostringstream os;
-        os << "unknown command: " << buf_com.str() << endl;
-        throw parse_error(os.str());
+            calculate_sorted_cells(m_context, sorted_cells, m_thread_count);
+            break;
+        }
+        case commands::type::check:
+        {
+            // Check cell results.
+            check();
+            break;
+        }
+        case commands::type::exit:
+        {
+            // Exit the loop.
+            m_parse_mode = parse_mode_exit;
+            return;
+        }
+        case commands::type::push:
+        {
+            switch (m_parse_mode)
+            {
+                case parse_mode_table:
+                    push_table();
+                    break;
+                case parse_mode_named_expression:
+                    push_named_expression();
+                    break;
+                default:
+                    throw parse_error("push command was used for wrong mode!");
+            }
+            break;
+        }
+        case commands::type::mode_init:
+        {
+            print_section_title("initializing");
+
+            m_parse_mode = parse_mode_init;
+            m_print_separator = true;
+            break;
+        }
+        case commands::type::mode_result:
+        {
+            // Clear any previous result values.
+            m_formula_results.clear();
+            m_parse_mode = parse_mode_result;
+            break;
+        }
+        case commands::type::mode_result_cache:
+        {
+            print_section_title("caching formula results");
+
+            m_parse_mode = parse_mode_result_cache;
+            m_print_separator = true;
+            break;
+        }
+        case commands::type::mode_edit:
+        {
+            print_section_title("editing");
+
+            m_parse_mode = parse_mode_edit;
+            m_dirty_formula_cells.clear();
+            m_modified_cells.clear();
+            m_print_separator = true;
+            break;
+        }
+        case commands::type::mode_table:
+        {
+            m_parse_mode = parse_mode_table;
+            mp_table_entry.reset(new table_handler::entry);
+            break;
+        }
+        case commands::type::mode_session:
+        {
+            print_section_title("session");
+
+            m_print_separator = true;
+            m_parse_mode = parse_mode_session;
+            break;
+        }
+        case commands::type::mode_named_expression:
+        {
+            m_print_separator = true;
+            m_parse_mode = parse_mode_named_expression;
+            mp_named_expression = ixion::make_unique<named_expression_type>();
+            break;
+        }
+        case commands::type::print_dependency:
+        {
+            print_section_title("print dependency");
+            print_dependency();
+            break;
+        }
+        case commands::type::unknown:
+        {
+            ostringstream os;
+            os << "unknown command: " << buf_cmd.str() << endl;
+            throw parse_error(os.str());
+        }
+        default:
+            ;
     }
 }
 
@@ -332,13 +426,21 @@ void model_parser::parse_session()
     }
 
     if (cmd == "row-limit")
-        m_row_limit = to_long(value);
+    {
+        rc_size_t ss = m_context.get_sheet_size();
+        ss.row = to_long(value);
+        m_context.set_sheet_size(ss);
+    }
     else if (cmd == "column-limit")
-        m_col_limit = to_long(value);
+    {
+        rc_size_t ss = m_context.get_sheet_size();
+        ss.column = to_long(value);
+        m_context.set_sheet_size(ss);
+    }
     else if (cmd == "insert-sheet")
     {
-        m_context.append_sheet(value.get(), value.size(), m_row_limit, m_col_limit);
-        cout << "sheet: (name: " << value << ", rows: " << m_row_limit << ", columns: " << m_col_limit << ")" << endl;
+        m_context.append_sheet(value.get(), value.size());
+        cout << "sheet: (name: " << value << ")" << endl;
     }
     else if (cmd == "current-sheet")
     {
@@ -379,17 +481,17 @@ void model_parser::parse_init()
                 m_context, pos, *mp_name_resolver, cell_def.value.get(), cell_def.value.size());
 
         m_context.set_grouped_formula_cells(cell_def.pos, std::move(tokens));
-        m_dirty_cells.insert(pos);
+        m_dirty_formula_cells.insert(cell_def.pos);
 
         cout << "{" << get_display_range_string(cell_def.pos) << "}: (m) " << cell_def.value.str() << endl;
         return;
     }
 
-    abs_address_iterator iter(cell_def.pos, abs_address_iterator::direction_type::vertical);
+    abs_address_iterator iter(cell_def.pos, rc_direction_t::vertical);
 
     for (const abs_address_t& pos : iter)
     {
-        m_dirty_cell_addrs.push_back(pos);
+        m_modified_cells.insert(pos);
 
         switch (cell_def.type)
         {
@@ -399,8 +501,10 @@ void model_parser::parse_init()
                     parse_formula_string(
                         m_context, pos, *mp_name_resolver, cell_def.value.get(), cell_def.value.size());
 
-                m_context.set_formula_cell(pos, std::move(tokens));
-                m_dirty_cells.insert(pos);
+                auto ts = formula_tokens_store::create();
+                ts->get() = std::move(tokens);
+                m_context.set_formula_cell(pos, ts);
+                m_dirty_formula_cells.insert(pos);
 
                 cout << get_display_cell_string(pos) << ": (f) " << cell_def.value.str() << endl;
                 break;
@@ -445,7 +549,7 @@ void model_parser::parse_edit()
         assert(cell_def.type == ct_formula);
         const abs_address_t& pos = cell_def.pos.first;
 
-        m_dirty_cell_addrs.push_back(pos);
+        m_modified_cells.insert(pos);
         unregister_formula_cell(m_context, pos);
 
         formula_tokens_t tokens =
@@ -453,23 +557,23 @@ void model_parser::parse_edit()
                 m_context, pos, *mp_name_resolver, cell_def.value.get(), cell_def.value.size());
 
         m_context.set_grouped_formula_cells(cell_def.pos, std::move(tokens));
-        m_dirty_cells.insert(pos);
+        m_dirty_formula_cells.insert(cell_def.pos);
         register_formula_cell(m_context, pos);
         return;
     }
 
-    abs_address_iterator iter(cell_def.pos, abs_address_iterator::direction_type::vertical);
+    abs_address_iterator iter(cell_def.pos, rc_direction_t::vertical);
 
     for (const abs_address_t& pos : iter)
     {
-        m_dirty_cell_addrs.push_back(pos);
+        m_modified_cells.insert(pos);
         unregister_formula_cell(m_context, pos);
 
         if (cell_def.value.empty())
         {
             // A valid name is given but with empty definition.  Just remove the
             // existing cell.
-            m_context.erase_cell(pos);
+            m_context.empty_cell(pos);
             continue;
         }
 
@@ -481,8 +585,10 @@ void model_parser::parse_edit()
                     parse_formula_string(
                         m_context, pos, *mp_name_resolver, cell_def.value.get(), cell_def.value.size());
 
-                m_context.set_formula_cell(pos, std::move(tokens));
-                m_dirty_cells.insert(pos);
+                auto ts = formula_tokens_store::create();
+                ts->get() = std::move(tokens);
+                m_context.set_formula_cell(pos, ts);
+                m_dirty_formula_cells.insert(pos);
                 register_formula_cell(m_context, pos);
                 cout << get_display_cell_string(pos) << ": (f) " << cell_def.value.str() << endl;
             }
@@ -527,6 +633,47 @@ void model_parser::parse_result()
         itr->second = fres;
 }
 
+void model_parser::parse_result_cache()
+{
+    parsed_assignment_type res = parse_assignment();
+
+    string name_s = res.first.str();
+
+    formula_result fres;
+    fres.parse(m_context, res.second.get(), res.second.size());
+
+    formula_name_t fnt = mp_name_resolver->resolve(
+        name_s.data(), name_s.size(), abs_address_t(m_current_sheet,0,0));
+
+    switch (fnt.type)
+    {
+        case formula_name_t::cell_reference:
+        {
+            abs_address_t pos = to_address(fnt.address).to_abs(abs_address_t());
+            formula_cell* fc = m_context.get_formula_cell(pos);
+            if (!fc)
+            {
+                std::ostringstream os;
+                os << name_s << " is not a formula cell";
+                throw model_parser::parse_error(name_s);
+            }
+
+            fc->set_result_cache(fres);
+
+            cout << get_display_cell_string(pos) << ": " << fres.str(m_context) << endl;
+            break;
+        }
+        case formula_name_t::range_reference:
+            throw model_parser::parse_error("TODO: we do not support setting result cache to range just yet.");
+        default:
+        {
+            std::ostringstream os;
+            os << "invalid cell name: " << name_s;
+            throw model_parser::parse_error(os.str());
+        }
+    }
+}
+
 void model_parser::parse_table()
 {
     assert(mp_table_entry);
@@ -633,15 +780,13 @@ void model_parser::push_named_expression()
 {
     assert(mp_named_expression);
 
-    std::unique_ptr<formula_tokens_t> tokens =
-        ixion::make_unique<formula_tokens_t>(
-            parse_formula_string(
-                m_context, mp_named_expression->origin, *mp_name_resolver,
-                mp_named_expression->expression.data(),
-                mp_named_expression->expression.size()));
+    formula_tokens_t tokens = parse_formula_string(
+        m_context, mp_named_expression->origin, *mp_name_resolver,
+        mp_named_expression->expression.data(),
+        mp_named_expression->expression.size());
 
     std::string exp_s = print_formula_tokens(
-        m_context, mp_named_expression->origin, *mp_name_resolver, *tokens);
+        m_context, mp_named_expression->origin, *mp_name_resolver, tokens);
 
     cout << "name: " << mp_named_expression->name << endl;
     cout << "expression: " << exp_s << endl;
@@ -683,6 +828,12 @@ void model_parser::push_named_expression()
     mp_named_expression.reset();
 }
 
+void model_parser::print_dependency()
+{
+    std::cout << get_formula_result_output_separator() << std::endl;
+    std::cout << m_context.get_cell_tracker().to_string() << std::endl;
+}
+
 void model_parser::parse_table_columns(const mem_str_buf& str)
 {
     assert(mp_table_entry);
@@ -967,7 +1118,7 @@ void model_parser::check()
             throw check_error("empty cell name");
 
         const formula_result& res = itr->second;
-        cout << name << " : " << res.str(m_context) << endl;
+        cout << name << ": " << res.str(m_context) << endl;
 
         formula_name_t name_type = mp_name_resolver->resolve(&name[0], name.size(), abs_address_t());
         if (name_type.type != formula_name_t::cell_reference)
@@ -979,13 +1130,13 @@ void model_parser::check()
 
         const formula_name_t::address_type& _addr = name_type.address;
         abs_address_t addr(_addr.sheet, _addr.row, _addr.col);
+        cell_access ca = m_context.get_cell_access(addr);
 
-        switch (m_context.get_celltype(addr))
+        switch (ca.get_type())
         {
             case celltype_t::formula:
             {
-                const formula_cell* fcell = m_context.get_formula_cell(addr);
-                formula_result res_cell = fcell->get_result_cache();
+                formula_result res_cell = ca.get_formula_result();
 
                 if (res_cell != res)
                 {
@@ -997,7 +1148,7 @@ void model_parser::check()
             }
             case celltype_t::numeric:
             {
-                double actual_val = m_context.get_numeric_value(addr);
+                double actual_val = ca.get_numeric_value();
                 if (actual_val != res.get_value())
                 {
                     ostringstream os;
@@ -1008,7 +1159,7 @@ void model_parser::check()
             }
             case celltype_t::boolean:
             {
-                bool actual = m_context.get_boolean_value(addr);
+                bool actual = ca.get_boolean_value();
                 bool expected = res.get_value() ? true : false;
                 if (actual != expected)
                 {
@@ -1020,26 +1171,18 @@ void model_parser::check()
             }
             case celltype_t::string:
             {
-                string_id_t str_id = m_context.get_string_identifier(addr);
+                const std::string* actual = ca.get_string_value();
+                if (!actual)
+                    throw check_error("failed to retrieve a string value from the cell.");
 
-                if (str_id != res.get_string())
+                const std::string& s_expected = res.get_string();
+                if (*actual != s_expected)
                 {
-                    const string* ps = m_context.get_string(str_id);
-                    if (!ps)
-                        throw check_error("failed to retrieve a string value for a string cell.");
-
-                    const string* ps_expected = m_context.get_string(res.get_string());
-                    if (!ps_expected)
-                    {
-                        std::ostringstream os;
-                        os << "no string value for the string ID of " << res.get_string();
-                        throw check_error(os.str());
-                    }
-
-                    ostringstream os;
-                    os << "unexpected string result: (expected: '" << *ps_expected << "'; actual: '" << *ps << "')";
+                    std::ostringstream os;
+                    os << "unexpected string result: (expected: '" << s_expected << "'; actual: '" << *actual << "')";
                     throw check_error(os.str());
                 }
+
                 break;
             }
             default:
