@@ -11,6 +11,7 @@
 
 #include "ixion/formula.hpp"
 #include "ixion/exceptions.hpp"
+#include "ixion/config.hpp"
 
 #include <iostream>
 #include <vector>
@@ -104,7 +105,7 @@ PyObject* document_append_sheet(pyobj_document* self, PyObject* args)
     ixion::model_context& cxt = sd->m_global->m_cxt;
     try
     {
-        sd->m_sheet_index = cxt.append_sheet(sheet_name, strlen(sheet_name), 1048576, 1024);
+        sd->m_sheet_index = cxt.append_sheet(sheet_name, strlen(sheet_name));
     }
     catch (const model_context_error& e)
     {
@@ -160,14 +161,15 @@ PyObject* document_calculate(pyobj_document* self, PyObject* args, PyObject* kwa
         return nullptr;
     }
 
-    model_context& cxt = self->m_data->m_global.m_cxt;
-    modified_cells_t& mod_cells = self->m_data->m_global.m_modified_cells;
-    dirty_formula_cells_t& dirty_fcells = self->m_data->m_global.m_dirty_formula_cells;
+    document_global& dg = self->m_data->m_global;
 
-    ixion::get_all_dirty_cells(cxt, mod_cells, dirty_fcells);
-    calculate_cells(cxt, dirty_fcells, threads);
-    mod_cells.clear();
-    dirty_fcells.clear();
+    // Query additional dirty formula cells and add them to the current set.
+    std::vector<abs_range_t> sorted = ixion::query_and_sort_dirty_cells(
+        dg.m_cxt, dg.m_modified_cells, &dg.m_dirty_formula_cells);
+    ixion::calculate_sorted_cells(dg.m_cxt, sorted, threads);
+
+    dg.m_modified_cells.clear();
+    dg.m_dirty_formula_cells.clear();
 
     Py_INCREF(Py_None);
     return Py_None;
