@@ -95,7 +95,7 @@ PyObject* sheet_set_numeric_cell(sheet* self, PyObject* args, PyObject* kwargs)
 
     ixion::model_context& cxt = sd->m_global->m_cxt;
     ixion::abs_address_t pos(sd->m_sheet_index, row, col);
-    sd->m_global->m_modified_cells.push_back(pos);
+    sd->m_global->m_modified_cells.insert(pos);
     cxt.set_numeric_cell(pos, val);
 
     Py_INCREF(Py_None);
@@ -122,7 +122,7 @@ PyObject* sheet_set_string_cell(sheet* self, PyObject* args, PyObject* kwargs)
 
     ixion::model_context& cxt = sd->m_global->m_cxt;
     ixion::abs_address_t pos(sd->m_sheet_index, row, col);
-    sd->m_global->m_modified_cells.push_back(pos);
+    sd->m_global->m_modified_cells.insert(pos);
     cxt.set_string_cell(pos, val, strlen(val));
 
     Py_INCREF(Py_None);
@@ -150,14 +150,15 @@ PyObject* sheet_set_formula_cell(sheet* self, PyObject* args, PyObject* kwargs)
     ixion::model_context& cxt = sd->m_global->m_cxt;
 
     ixion::abs_address_t pos(sd->m_sheet_index, row, col);
-    sd->m_global->m_modified_cells.push_back(pos);
     sd->m_global->m_dirty_formula_cells.insert(pos);
 
     ixion::formula_tokens_t tokens =
         ixion::parse_formula_string(
             cxt, pos, *sd->m_global->m_resolver, formula, strlen(formula));
 
-    cxt.set_formula_cell(pos, std::move(tokens));
+    auto ts = formula_tokens_store::create();
+    ts->get() = std::move(tokens);
+    cxt.set_formula_cell(pos, ts);
 
     // Put this formula cell in a dependency chain.
     ixion::register_formula_cell(cxt, pos);
@@ -187,7 +188,7 @@ PyObject* sheet_get_numeric_value(sheet* self, PyObject* args, PyObject* kwargs)
     double val = 0.0;
     try
     {
-        val = cxt.get_numeric_value_nowait(ixion::abs_address_t(sd->m_sheet_index, row, col));
+        val = cxt.get_numeric_value(ixion::abs_address_t(sd->m_sheet_index, row, col));
     }
     catch (const formula_error&)
     {
@@ -216,14 +217,9 @@ PyObject* sheet_get_string_value(sheet* self, PyObject* args, PyObject* kwargs)
     }
 
     ixion::model_context& cxt = sd->m_global->m_cxt;
-    string_id_t sid = cxt.get_string_identifier_nowait(ixion::abs_address_t(sd->m_sheet_index, row, col));
-    const std::string* ps = cxt.get_string(sid);
+    const std::string* ps = cxt.get_string_value(ixion::abs_address_t(sd->m_sheet_index, row, col));
     if (!ps)
-    {
-        PyErr_SetString(PyExc_RuntimeError,
-            "Failed to get a string object associated with a string ID.");
-        return nullptr;
-    }
+        return PyUnicode_FromStringAndSize(nullptr, 0);
 
     return PyUnicode_FromStringAndSize(ps->data(), ps->size());
 }
@@ -265,6 +261,12 @@ PyObject* sheet_get_formula_expression(sheet* self, PyObject* args, PyObject* kw
 }
 
 PyObject* sheet_erase_cell(sheet* self, PyObject* args, PyObject* kwargs)
+{
+    PyErr_SetString(PyExc_RuntimeError, "erase_cell() method has been deprecated. Please use empty_cell() instead.");
+    return nullptr;
+}
+
+PyObject* sheet_empty_cell(sheet* self, PyObject* args, PyObject* kwargs)
 {
     int col = -1;
     int row = -1;
@@ -283,8 +285,8 @@ PyObject* sheet_erase_cell(sheet* self, PyObject* args, PyObject* kwargs)
 
     ixion::model_context& cxt = sd->m_global->m_cxt;
     abs_address_t pos(sd->m_sheet_index, row, col);
-    sd->m_global->m_modified_cells.push_back(pos);
-    cxt.erase_cell(pos);
+    sd->m_global->m_modified_cells.insert(pos);
+    cxt.empty_cell(pos);
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -298,6 +300,7 @@ PyMethodDef sheet_methods[] =
     { "get_numeric_value", (PyCFunction)sheet_get_numeric_value, METH_VARARGS | METH_KEYWORDS, "get numeric value from specified cell" },
     { "get_string_value",  (PyCFunction)sheet_get_string_value,  METH_VARARGS | METH_KEYWORDS, "get string value from specified cell" },
     { "get_formula_expression", (PyCFunction)sheet_get_formula_expression, METH_VARARGS | METH_KEYWORDS, "get formula expression string from specified cell position" },
+    { "empty_cell", (PyCFunction)sheet_empty_cell, METH_VARARGS | METH_KEYWORDS, "empty cell at specified position" },
     { "erase_cell", (PyCFunction)sheet_erase_cell, METH_VARARGS | METH_KEYWORDS, "erase cell at specified position" },
     { nullptr }
 };
