@@ -8,10 +8,11 @@
 #ifndef __IXION_SORT_INPUT_PARSER_HXX__
 #define __IXION_SORT_INPUT_PARSER_HXX__
 
-#include "ixion/depth_first_search.hpp"
 #include "ixion/mem_str_buf.hpp"
 #include "ixion/exceptions.hpp"
 
+#include "depth_first_search.hpp"
+
 #include <vector>
 
 namespace ixion {
@@ -25,16 +26,7 @@ class sort_input_parser
         virtual ~parse_error() throw();
     };
 
-    class cell_handler : public ::std::unary_function<mem_str_buf, void>
-    {
-    public:
-        cell_handler(::std::vector<mem_str_buf>& sorted);
-        void operator() (const mem_str_buf& s);
-    private:
-        ::std::vector<mem_str_buf>& m_sorted;
-    };
-
-    typedef depth_first_search<mem_str_buf, cell_handler, mem_str_buf::hash> dfs_type;
+    using dfs_type = depth_first_search<mem_str_buf, mem_str_buf::hash>;
 
 public:
     sort_input_parser(const ::std::string& filepath);
@@ -48,7 +40,7 @@ private:
     void insert_depend(const mem_str_buf& cell, const mem_str_buf& dep);
 
 private:
-    dfs_type::precedent_set    m_set;
+    dfs_type::relations    m_set;
     ::std::string           m_content;
     ::std::vector<mem_str_buf>   m_all_cells;
 
